from django.conf.urls.defaults import patterns, include, url

# Uncomment the next two lines to enable the admin:

urlpatterns = patterns('',

    url(r'^payment/dibs/', include('cratis.app.ecommerce.payment.dibs.urls')),
    url(r'^payment/paypal/', include('cratis.app.ecommerce.payment.paypal.urls')),
    url(r'^payment/pangalink/', include('cratis.app.ecommerce.payment.pangalink.urls')),
    url(r'^payment/paytrail/', include('cratis.app.ecommerce.payment.paytrail.urls')),
    url(r'^payment/kk/', include('cratis.app.ecommerce.payment.kardikeskus.urls')),
    url(r'^payment/voucher/', include('cratis.app.ecommerce.payment.voucher.urls')),
)



from django.conf.urls.defaults import patterns, include, url

urlpatterns = patterns('',
    url(r'^cart/delivery/(?P<product_id>\d+)/(?P<method>[a-zA-Z0-9\-_]+)$', 'cratis.app.ecommerce.shopping_cart.views.set_delivery', name='cart_delivery'),
    url(r'^cart/qty/(?P<product_id>\d+)/(?P<quantity>\d+)$', 'cratis.app.ecommerce.shopping_cart.views.set_product_qty', name='cart_qty'),
    url(r'^cart/add/(?P<product_id>\d+)/(?P<quantity>\d+)$', 'cratis.app.ecommerce.shopping_cart.views.add_to_cart', name='cart_add'),
    url(r'^cart/remove/(?P<product_id>\d+)/(?P<quantity>\d+)$', 'cratis.app.ecommerce.shopping_cart.views.remove_from_cart', name='cart_remove'),
    url(r'^cart/clear$', 'cratis.app.ecommerce.shopping_cart.views.clear_cart', name='cart_clear'),
)



"""
Default settings for cratis project
"""
import logging
import os

from configurations import Configuration, values
import sys


class Common(Configuration):

    @classmethod
    def load_features(cls):
        loaded_features = []
        for feature in cls.FEATURES:

            for requirement in feature.get_required(cls):
                if not requirement in loaded_features:
                    message = '\n Feature %s depends on %s, that is not loaded yet. Order is also important!\n' % (
                        str(type(feature)),
                        str(requirement),
                    )
                    print(message)
                    sys.exit(1)

            feature.configure_settings(cls)

            loaded_features.append(type(feature))

    @classmethod
    def pre_setup(cls):
        super(Common, cls).pre_setup()

        cls.load_features()

        if len(sys.argv) == 2 and sys.argv[1] == '--requirements':

            all_requirements = []

            for feature in cls.FEATURES:
                requirements = []
                for req in feature.get_required_packages(cls):
                    if not req in all_requirements:
                        requirements.append(req)
                        all_requirements.append(req)

                if requirements:
                    print('# %s\n%s\n' % (feature.__class__.__name__, '\n'.join(requirements)))

            exit()


    # Build paths inside the project like this: os.path.join(BASE_DIR, ...)
    BASE_DIR = os.environ['CRATIS_APP_PATH']

    # Quick-start development settings - unsuitable for production
    # See https://docs.djangoproject.com/en/{{ docs_version }}/howto/deployment/checklist/

    # SECURITY WARNING: keep the secret key used in production secret!
    SECRET_KEY = 'testing_only'

    # SECURITY WARNING: don't run with debug turned on in production!
    DEBUG = True

    TEMPLATE_DEBUG = True

    ALLOWED_HOSTS = []

    # Application definition

    INSTALLED_APPS = (
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.messages',
        'django.contrib.staticfiles',
        'django.contrib.sites',
        'south',
        'cratis.app.utils',
        'django_wsgiserver'
    )

    MIDDLEWARE_CLASSES = (
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',
    )

    ROOT_URLCONF = 'cratis.urls'

    # WSGI_APPLICATION = '{{ project_name }}.wsgi.application'


    # Database
    # https://docs.djangoproject.com/en/{{ docs_version }}/ref/settings/#databases
    # http://django-configurations.readthedocs.org/en/latest/values/#configurations.values.DatabaseURLValue

    DATABASES = values.DatabaseURLValue('sqlite://%s' % (os.sep + os.path.join(BASE_DIR, 'var', 'db.sqlite3')),
                                        environ=True)

    # Internationalization
    # https://docs.djangoproject.com/en/{{ docs_version }}/topics/utils/

    TIME_ZONE = 'UTC'
    USE_TZ = True

    # Static files (CSS, JavaScript, Images)
    # https://docs.djangoproject.com/en/{{ docs_version }}/howto/static-files/

    LOCALE_PATHS = (
        BASE_DIR + '/locale',
    )

    STATIC_URL = '/static/'
    STATIC_ROOT = BASE_DIR + '/var/static'

    MEDIA_URL = '/media/'
    MEDIA_ROOT = BASE_DIR + '/var/media'

    STATICFILES_DIRS = ()
    TEMPLATE_DIRS = ()

    FEATURES = ()
    SITE_ID = 1


class CratisDev(Common):
    """
    The in-development settings and the default configuration.
    """
    pass


class CratisProd(Common):
    """
    The in-production settings.
    """
    DEBUG = False
    TEMPLATE_DEBUG = DEBUG

    SECRET_KEY = values.SecretValue()

Dev = CratisDev
Prod = CratisProd


# import cms
# from cms.views import details

from django.conf.urls import patterns, include, url
from django.conf import settings


urlpatterns = patterns('',)

if settings.DEBUG:
    urlpatterns += patterns('',
        url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT}),
        url(r'^assets/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.STATIC_ROOT}),
    )

for feature in settings.FEATURES:
    feature.configure_urls(settings, urlpatterns)




from models import Item
from models import Cart
from cratis.app.core.common.i18n import message_wrapper
from django.contrib import admin


#class ProductCategoryAdmin(AdminImageMixin, admin.ModelAdmin):
#    pass
#

class InlineProduct(admin.TabularInline):
    model = Item
    extra = 0

class CartAdmin(admin.ModelAdmin):
    inlines = [InlineProduct]

    def queryset(self, request):
        qs = super(CartAdmin, self).queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(author=request.user)

    def has_add_permission(self, request):
        return False




#    class Media:
#        js = ("js/admin/product_form.js",)

admin.site.register(Cart, CartAdmin)

#admin.site.register(Product, ProductAdmin)
#admin.site.register(ProductCategory, ProductCategoryAdmin)

#admin.site.register = I18nLabel(admin.site.register).register()
#admin.site.app_index = I18nLabel(admin.site.app_index).index()
admin.ModelAdmin.message_user = message_wrapper(admin.ModelAdmin.message_user)


import base64
import os
import re

__author__ = 'alex'

from M2Crypto import BIO, RSA, EVP, RC4, X509


def _cleanup_key(key):

    if isinstance(key, unicode):
        key = bytes(key)

    key = key.strip()
    key = re.sub('\s*[\r\n]+\s*', '\n', key)
    key = re.sub('^\s*$', '', key, re.MULTILINE)
    key = re.sub('\n+', '\n', key)
    key = re.sub('^\s+', '', key, re.MULTILINE)
    return key

def load_pem_private_key(privateKey):
#    bio =
#    return RSA.load_key_bio(BIO.MemoryBuffer(privateKey))
    return privateKey


def load_pem_public_key(publicKey):
    bio = BIO.MemoryBuffer(_cleanup_key(publicKey))
    return RSA.load_pub_key_bio(bio)

def load_pem_cert_public_key(cert):
    bio = BIO.MemoryBuffer(_cleanup_key(cert))
    return X509.load_cert_bio(bio).get_pubkey().get_rsa()

def ssl_sign(string, privateKey):
    signEvp = EVP.load_key_string(_cleanup_key(privateKey))
    signEvp.sign_init()
    signEvp.sign_update(string)
    return signEvp.sign_final()

def ssl_verify(sting, signature, pubkey):
    verifyEVP = EVP.PKey()
    verifyEVP.assign_rsa(pubkey)
    verifyEVP.verify_init()
    verifyEVP.verify_update(sting)
    return verifyEVP.verify_final(signature) == 1

def rsa_public2private_encrypt(data, publicKey):
    return publicKey.public_encrypt(data, RSA.pkcs1_padding)

def rsa_public2private_decrypt(data, privateKey):
    return RSA.load_key_bio(BIO.MemoryBuffer(privateKey)).private_decrypt(data, RSA.pkcs1_padding)

def rc4_encrypt(data, key):
    rc4 = RC4.RC4()
    rc4.set_key(key)
    return rc4.update(data) + rc4.final()

def rc4_decrypt(data, key):
    return rc4_encrypt(data, key) # second time encryption decrypts the data


def seal(string, pubkey):
    randomBytes = os.urandom(16)
    encryptedRC4Key = rsa_public2private_encrypt(randomBytes, pubkey)
    encryptedData =  rc4_encrypt(string, randomBytes)

    return base64.b64encode(encryptedData) + '@' + base64.b64encode(encryptedRC4Key)

def unseal(string, privateKey):
    randomBytes = os.urandom(16)

    (cipherText, cipherKey) = string.split('@')
    decryptedKey = rsa_public2private_decrypt(base64.b64decode(cipherKey), privateKey)
    decryptedData =  rc4_encrypt(base64.b64decode(cipherText), decryptedKey)

    return decryptedData



def cart_context(request):
    """
    @type context: HttpRequest
    """

    if request.path[0:7] == '/admin/':
        return {}

    from cratis.app.ecommerce.shopping_cart.cart import Cart

    context = {
        'shoping_cart': Cart(request)
    }
    return context


from voluptuous import Schema
from cratis.app.ecommerce.payment.common import Payment

__author__ = 'alex'



class DibsPayment(Payment):

    actions_template = 'payment/dibs_actions.html'

    def schema(self):
        return Schema({
            'MD51': str,
            'MD52': str,
            'CREDITCHOICES': [str],
            'MERCHANT': str,
            'LIVE': bool,
            'CALLBACK_DOMAIN': str,
            'CURRENCY': str
        }, required=True)


    def require_payment(self):
        return True







import re
from django.core.exceptions import ObjectDoesNotExist
from django.utils.functional import lazy
from django.conf import settings

from django.utils import translation

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _, get_language, ugettext
from django.utils.text import capfirst
from django.db.models.base import ModelBase


def localize_pattern(text):
    """
    Generates proper prefixse for urls and
    transletes the rest.

    Wrap route with this function if you need translatable url.

    @param text:
    @return:
    """

    translated = ugettext(text)
    language = get_language()

    if language != settings.MAIN_LANGUAGE:
        result = '^' + language + '/' + translated[1:]
    else:
        result = translated

#    print "[%s : %s] : %s {%s}" % (language, settings.MAIN_LANGUAGE, text, result, )
    return result

localize_url = lazy(localize_pattern, unicode)


class I18nLabel():
    def __init__(self, function):
        self.target = function
        self.app_label = u''

    def rename(self, f, name = u''):
        def wrapper(*args, **kwargs):
            extra_context = kwargs.get('extra_context', {})
            if 'delete_view' != f.__name__:
                extra_context['title'] = self.get_title_by_name(f.__name__, args[1], name)
            else:
                extra_context['object_name'] = name.lower()
            kwargs['extra_context'] = extra_context
            return f(*args, **kwargs)
        return wrapper

    def get_title_by_name(self, name, request={}, obj_name = u''):
        if 'add_view' == name:
            return _('Add %s') % obj_name
        elif 'change_view' == name:
            return _('Change %s') % obj_name
        elif 'changelist_view' == name:
            if 'pop' in request.GET:
                title = _('Select %s')
            else:
                title = _('Select %s to change')
            return title % obj_name
        else:
            return ''

    def wrapper_register(self, model_or_iterable, admin_class=None, **option):
        if isinstance(model_or_iterable, ModelBase):
            model_or_iterable = [model_or_iterable]
        for model in model_or_iterable:
            if admin_class is None:
                admin_class = type(model.__name__+'Admin', (admin.ModelAdmin,), {})
            self.app_label = model._meta.app_label
            current_name = model._meta.verbose_name.upper()
            admin_class.add_view = self.rename(admin_class.add_view, current_name)
            admin_class.change_view = self.rename(admin_class.change_view, current_name)
            admin_class.changelist_view = self.rename(admin_class.changelist_view, current_name)
            admin_class.delete_view = self.rename(admin_class.delete_view, current_name)
        return self.target(model, admin_class, **option)

    def wrapper_app_index(self, request, app_label, extra_context=None):
        if extra_context is None:
                extra_context = {}
        extra_context['title'] = _('%s administration') % _(capfirst(app_label))
        return self.target(request, app_label, extra_context)

    def register(self):
        return self.wrapper_register

    def index(self):
        return self.wrapper_app_index


def message_wrapper(f):
    def wrapper(self, request, message):

        return f(self, request, message)

    return wrapper


def _try_to_find_by_localized_field(subject_class, trans_class, lang, args):
    try:
        if lang == settings.MAIN_LANGUAGE:
            return subject_class.objects.get(**args)
        else:
            args['lang'] = lang
            return trans_class.objects.get(**args).related
    except ObjectDoesNotExist:
        return None


def find_by_localized_field(subject_class, field_name, field_value, lang=None):
    args = {field_name: field_value}

    if not lang:
        lang = get_language()

    trans_class = subject_class.get_translation_class()
    result = _try_to_find_by_localized_field(subject_class, trans_class, lang, args)

    if result:
        return result

    # try another languages
    for code, name in settings.LANGUAGES:
        if lang != code: # already tried
            result = _try_to_find_by_localized_field(subject_class, trans_class, code, {field_name: field_value})
            if result:
                return result

    return None



from django.conf.urls import patterns
from cratis.features import Feature


class WebShop(Feature):
    def configure_settings(self, cls):
        super(WebShop, self).configure_settings(cls)

        self.append_apps(cls, [
            # "cratis.app.ecommerce.shopping_cart",
            # "cratis.app.ecommerce.payment",
            "cratis.app.ecommerce.products",
            # "cratis.app.ecommerce.orders"
        ])

        self.append_template_processor(cls, [
            # "cratis.app.ecommerce.products.context.shop_context",
            # "cratis.app.ecommerce.shopping_cart.context.cart_context",
        ])

    def configure_urls(self, cls, urls):
        super(WebShop, self).configure_urls(cls, urls)

        # urls += patterns('',
            # (r'^$', "cratis.app.ecommerce.shopping_cart.urls"),
            # (r'^$', "cratis.app.ecommerce.shops.urls"),
            # (r'^$', "cratis.app.ecommerce.orders.urls"),
        # )



import json
from django.core.urlresolvers import reverse
from django.forms.models import ModelForm
from django.http import HttpResponseRedirect, HttpResponse
from django.views.decorators.http import require_POST
from django.views.generic.base import TemplateView
from django.views.generic.detail import DetailView
from django.views.generic.edit import CreateView, ModelFormMixin, UpdateView, DeleteView
from django.views.generic.list import ListView
from cratis.app.core.common.i18n import find_by_localized_field
from cratis.app.ecommerce.orders.models import Address, Order
from cratis.app.ecommerce.profile.models import UserProfile
from cratis.app.ecommerce.products.models import ProductCategory

__author__ = 'Alex'


class WithShopingCartMixin(object):

    def get_context_data(self, **kwargs):
        context = super(WithShopingCartMixin, self).get_context_data(**kwargs)


        return context

class WithSidebarMixin(object):

    def get_context_data(self, **kwargs):
        context = super(WithSidebarMixin, self).get_context_data(**kwargs)

        if 'category_slug' in self.kwargs:
            loc_cat = find_by_localized_field(ProductCategory, 'slug', self.kwargs['category_slug'])
            if loc_cat:
                self.kwargs['category_id'] = loc_cat.id

        if 'category_id' in self.kwargs:
            context['cur_category'] = ProductCategory.objects.get(pk=self.kwargs['category_id'])

        return context

class Profile(WithShopingCartMixin, TemplateView):
    template_name= 'profile.html'

    def get_context_data(self, **kwargs):
        context = super(Profile, self).get_context_data(**kwargs)

        # address list
        profile = self.request.user.get_profile()

        context['profile'] = profile

        context['address_list'] = Address.objects.filter(profile=profile)


        context['orders_confirmed'] = Order.objects.filter(user=profile, status=Order.ORDER_STATUS_CONFIRMED)
        context['orders_paid'] = Order.objects.filter(user=profile, status=Order.ORDER_STATUS_PAID)
        context['orders_completed'] = Order.objects.filter(user=profile, status=Order.ORDER_STATUS_COMPLETED)
        context['orders_rejected'] = Order.objects.filter(user=profile, status=Order.ORDER_STATUS_REJECTED)

        return context


class ProfileAddressList(WithShopingCartMixin, ListView):

    template_name= 'profile/address_list.html'
    model = Address

    def get_queryset(self):
        q = super(ProfileAddressList, self).get_queryset()

        profile = self.request.user.get_profile()
        return q.filter(profile=profile)

class ProfileOrderList(WithShopingCartMixin, ListView):

    template_name= 'profile/order_list.html'
    model = Order

    def get_queryset(self):
        q = super(ProfileOrderList, self).get_queryset()

        profile = self.request.user.get_profile()
        return q.filter(user=profile)

class ProfileOrderDetails(WithShopingCartMixin, DetailView):

    template_name= 'profile/order_details.html'
    model = Order
    context_object_name = 'order'

    def get_queryset(self):
        q = super(ProfileOrderDetails, self).get_queryset()

        profile = self.request.user.get_profile()
        return q.filter(user=profile)
#
class AddressForm(ModelForm):
    class Meta:
        model = Address
        exclude = ('profile', 'temporary', 'is_main', 'recipient', 'country')

class ProfileAddressNew(WithShopingCartMixin, CreateView):
    template_name= 'profile/address_new.html'
    model = Address
    form_class = AddressForm

    def get_success_url(self):
        return reverse('profile')




    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.profile = self.request.user.get_profile()
        self.object.save()
        return super(ModelFormMixin, self).form_valid(form)

@require_POST
def update_address(request, id):

    address = Address.objects.get(pk=int(id))

    if not address or address.profile.pk != request.user.get_profile().pk:
        return HttpResponse(json.dumps({'ok':False, 'message': 'Not authorized'}), mimetype='application/javascript')


    if 'country' in request.POST:
        address.country = request.POST['country']
    address.city = request.POST['city']
    address.street_address = request.POST['street_address']
    address.lastname = request.POST['lastname']
    address.firstname = request.POST['firstname']
    address.post_index = request.POST['post_index']
    address.phone = request.POST['phone']
    address.email = request.POST['email']
    address.save()

    return HttpResponse(json.dumps({'ok':True}), mimetype='application/javascript')



def make_address_default(request, id):
    profile = request.user.get_profile()
    for address in profile.saved_adresses():
        address.is_main = (address.pk == int(id))
        address.save()

    return HttpResponseRedirect(reverse('profile'))



class ProfileAddressEdit(WithShopingCartMixin, UpdateView):
    template_name= 'profile/address_edit.html'
    model = Address
    form_class = AddressForm

    def get_queryset(self):
        q = super(ProfileAddressEdit, self).get_queryset()
        profile = self.request.user.get_profile()
        return q.filter(profile=profile)


    def get_success_url(self):
        return reverse('profile')

class ProfileForm(ModelForm):
    class Meta:
        model = UserProfile
        exclude = ('user',)

class ProfileEdit(WithShopingCartMixin, UpdateView):
    template_name= 'profile/edit.html'
    model = UserProfile
    form_class = ProfileForm

    def get_object(self, queryset=None):
        profile = self.request.user.get_profile()
        return profile

    def get_success_url(self):
        return reverse('profile')

class ProfileAddressRemove(WithShopingCartMixin, DeleteView):
    model = Address

    template_name = 'profile/address_confirm_delete.html'

    def get_queryset(self):
        q = super(ProfileAddressRemove, self).get_queryset()
        profile = self.request.user.get_profile()
        return q.filter(profile=profile)

    def get_success_url(self):
        return reverse('profile')











from cratis.app.core.common.templatetags.model_trans import localize

__author__ = 'Alex'


from django.core.urlresolvers import reverse
#from cratis.app.ecommerce.shopping_cart.cart import Cart

from django import template
register = template.Library()

@register.simple_tag()
def category_url(category):
    category_slug = localize(category, 'slug')
    if category_slug != '':
        return reverse('products_in_category_by_slug', kwargs={'category_slug': category_slug})
    else:
        return reverse('products_in_category', kwargs={'category_id': category.pk})

#@register.filter
#def shoping_cart(request):
#    return Cart(request)

@register.simple_tag()
def product_url(product, category=None):

    if not category and product.categories.count():
        category = product.categories.all()[0]

    product_slug = localize(product, 'slug')

    if not category:
        if product_slug != '':
            return reverse('product_by_slug', kwargs={'slug': product_slug})
        else:
            return reverse('product', kwargs={'pk': product.pk})

    else:
        category_slug = localize(category, 'slug')

        if category_slug != '' and product_slug != '':
            return reverse('product_with_cat_both_by_slug',
                kwargs={'slug': product_slug, 'category_slug': category_slug})

        elif category_slug != '':
            return reverse('product_with_cat_cat_by_slug_prod_by_id',
                kwargs={'pk': product.pk, 'category_slug': category_slug})

        elif product_slug != '':
            return reverse('product_with_cat_cat_by_id_prod_by_slug',
                kwargs={'slug': product_slug, 'category_id': category.pk})

        else:
            return reverse('product_with_cat',
                kwargs={'pk': product.pk, 'category_id': category.pk})





from django import forms
from voluptuous import Required, Schema
from cratis.app.ecommerce.payment.common import Payment
from django.utils.translation import ugettext as _

class VoucherForm(forms.Form):
    voucher_code = forms.CharField(max_length=100, label=_('Voucher code'))
    email = forms.CharField(max_length=100, label=_('Email'))
    phone = forms.CharField(max_length=100, label=_('Phone'))

class VoucherPayment(Payment):

    actions_template = 'payment/voucher_actions.html'

    def get_form(self, data):
        return VoucherForm(data)

    def actions_context(self):

        return {
            'form': self.get_form(None)
        }


    def schema(self):
        return Schema({
            'pkey': self.valid_key(),
            'pubkey': self.valid_key(),
            'account': str,
            'owner': basestring,
            'id': str,
            'url': str
        }, required=True)

    def require_payment(self):
        return True





# coding: utf-8
from voluptuous import Schema, MultipleInvalid
from cratis.features import Feature


class I18n(Feature):

    def __init__(self, langs=(('en', 'English'),), main_lang='en', admin_lang=None, fallback_translations=False):
        self.main_lang = main_lang
        self.admin_lang = admin_lang if admin_lang else main_lang
        self.langs = langs

        self.fallback_translations = fallback_translations

        try:
            Schema(((str, str),))(langs)
        except MultipleInvalid as e:
            self.report_failure('Parameter "langs" validation error: %s' % str(e))

    def configure_settings(self, cls):

        self.append_apps(cls, ('cratis.app.i18n',))

        cls.USE_FALLBACK_TRANSLATION = self.fallback_translations
        cls.LANGUAGE_CODE = self.main_lang
        cls.USE_I18N = True
        cls.USE_L10N = True

        if self.langs:
            cls.LANGUAGES = self.langs

        cls.MAIN_LANGUAGE = self.main_lang
        cls.ADMIN_LANGUAGE = self.admin_lang

        self.append_middleware(cls, ('cratis.app.i18n.middleware.LocaleRewriteMiddleware',))

        self.append_template_processor(cls, ('cratis.app.i18n.context.i18n_context',))

        cls.USE_FALLBACK_TRANSLATION = False


class I18nAdminUi(Feature):

    def get_required_packages(self, cls):
        return 'django-rosetta',

    def configure_settings(self, cls):
        self.append_apps(cls, [
            'rosetta',
        ])

        cls.ROSETTA_WSGI_AUTO_RELOAD = True
        cls.ROSETTA_MESSAGES_PER_PAGE = 20
        cls.ROSETTA_STORAGE_CLASS = 'rosetta.storage.SessionRosettaStorage'


    def configure_urls(self, cls, urls):
        from django.conf.urls import url, include, patterns

        urls += patterns('',
            url(r'^admin/i18n/', include('rosetta.urls')),
        )






from django.test import TestCase
from cratis.app.ecommerce.products.models import Product
from models import Cart, Item
from django.contrib.auth.models import User
import datetime
from decimal import Decimal


class CartAndItemModelsTestCase(TestCase):

    def test_create_cart_at_first_product(self):
        """
        Cart should be created only just before first product is added to it.

        :return:
        """
        carts = Cart.objects.all()
        assert len(carts) == 0

        self.client.get('/')

        carts = Cart.objects.all()
        assert len(carts) == 0

        self.client.get('/')

        product = Product()
        product.title = 'Test'
        product.price = 3.50
        product.save()

        self.client.get('/cart/add/%s/%d' % (product.pk, 1))

        carts = Cart.objects.all()
        assert len(carts) == 1




from django.conf import settings
import re
from django.utils import translation


class LocaleRewriteMiddleware(object):
    """
    settings should contain MAIN_LANGUAGE definition.
    """

    def process_response(self, request, response):
        if request.path[0:17] == '/admin/auth/user/':
            response.set_cookie('ad_lang', '1', 60 * 60 * 24 * 365)
        return response

    def process_request(self, request):
        if request.path[0:7] == '/admin/':
            lang = settings.ADMIN_LANGUAGE
        else:
            match = re.match('^/([a-z]{2})/', request.path)
            if match:
                lang = match.group(1)
            else:
                lang = settings.MAIN_LANGUAGE

        translation.activate(lang)
        request.LANGUAGE_CODE = translation.get_language()

        return None



import hashlib
import urllib
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, HttpResponse
from django.utils.encoding import smart_str
from django.utils.translation import get_language
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt
from cratis.app.core.settings import APP_SETTINGS
from cratis.app.ecommerce.orders.models import Order
from cratis.app.ecommerce.payment.common import log_payment, get_client_ip
from cratis.app.ecommerce.payment.views import PaymentView


def md5(string):
    return hashlib.md5(string).hexdigest()


def validate_payment(request):

    settings = config_get_group('PAYMENT_DIBS')
    transaction_id = request.POST['transact']
    amount = request.POST['amount']
    currency = request.POST['currency']

    hash_data = '&transact=' + smart_str(transaction_id) + '&amount=' + smart_str(amount) + '&currency=' + smart_str(currency)
    md5_key1 = settings['MD51']
    md5_key2 = settings['MD52']

    calculated_authkey = md5(md5_key2 + md5(md5_key1 + hash_data))
    if calculated_authkey != request.POST['authkey']:
        log_payment(request, 'dibs', 'pay_accept_error',
            'Siganture not valid. Data: ' + hash_data + ' Key: ' + calculated_authkey)
        return False
    return True


class DibsAccept(PaymentView):

    def post(self, request, *args, **kwargs):
        log_payment(request, 'dibs', 'pay_accept')

        # skip validation
        order_id = request.POST['orderid']
        print order_id
        order = Order.objects.get(pk=order_id)
        order.mark_paid()

        return HttpResponseRedirect(reverse('cratis.app.ecommerce.orders.views.confirm'))


        #    if not validate_payment(request):
        #        return HttpResponse('Bad signature. Can not accpet payment.', None, 403)
        #

        #
        #    amount = request.POST['amount']
        #
        #    order = Order.objects.get(order_id)
        #    if (order.get_detalisation()['full_total'] * 100) != amount:
        #        log_payment(request, 'dibs', 'pay_accept_error', 'Order amount do not match: %s not equals %s' % (order.get_detalisation()['full_total'] * 100, amount))

        #    return HttpResponse('pay_accept')


class DibsCancel(PaymentView):

    def post(self, request, *args, **kwargs):
        log_payment(request, 'dibs', 'pay_cancel')

    #    order_id = request.POST['orderid']
    #    order = Order.objects.get(order_id)
    #    order.mark_paid()

        return HttpResponseRedirect(reverse('orders_checkout'))


class DibsCallback(PaymentView):

    def post(self, request, *args, **kwargs):
        log_payment(request, 'dibs', 'pay_callback')

        # skip validation
        order_id = request.POST['orderid']
        order = Order.objects.get(order_id)
        order.mark_paid()

        return HttpResponse('ok')


class DibsStart(PaymentView):

    def get(self, request, *args, **kwargs):

        method = self.load_method(kwargs['method'])
        settings = method.behavior().config

        order = Order.objects.get(pk=request.session['order_id'])
    #    order.mark_confirmed()

        detalisation = order.get_detalisation()

    #    for row in detalisation['rows']:
    #        ulink_order.items.append(UlinkOrderItem(smart_str(row['title']), '', str(row['price'])))


        # Preparing the data that we are sending to DIBS
        # Order total to be sent to DIBS must be an int specified in cents or
        # equivalent.
        order_total = int(detalisation['full_total'] * 100)
        order_id = order.id

        # Create md5 hash to make payment secure:
        md5_key = md5(settings['MD52'] +
                        md5(settings['MD51'] + 'merchant=%s&orderid=%s&currency=%s&amount=%s' % (settings['MERCHANT'], order_id, settings['CURRENCY'], order_total)))

        # Create the cancel and accept url, based on the request to get the host
        # and reverse to get the url.
        #        cancelurl = 'http://' + request.META['HTTP_HOST'] + reverse('satchmo_checkout-step1')
        #        accepturl = 'http://' + request.META['HTTP_HOST'] + reverse('DIBS_satchmo_checkout-success')
        #        callbackurl = 'http://' + request.META['HTTP_HOST'] + reverse('DIBS_satchmo_checkout-step4') + '?order_id=' + str(order.id)
        #
        cancelurl = 'http://' + settings['CALLBACK_DOMAIN'] + reverse('dibs_payment_cancel', kwargs={'method': method.slug})
        accepturl = 'http://' + settings['CALLBACK_DOMAIN'] + reverse('dibs_payment_accept', kwargs={'method': method.slug})
        callbackurl = 'http://' + settings['CALLBACK_DOMAIN'] + reverse('dibs_payment_callback', kwargs={'method': method.slug})

        data = [
            ('merchant', settings['MERCHANT']),
            ('amount', order_total),
            ('currency', settings['CURRENCY']),
            ('orderid', order_id),
            ('accepturl', accepturl),
            ('cancelurl', cancelurl),
            ('callbackurl', callbackurl),
            ('ip', get_client_ip(request)),
            #('uniqueoid', 'yes'),
            ('lang', get_language()),
            ('md5key', md5_key),
            ('calcfee', 'yes'),
            # Currently not implemented in the flex window.
            # ('delivery1', order.ship_addressee),
            # ('delivery2', order.ship_street1),
            # ('delivery3',  order.ship_postal_code + ' ' +  order.ship_city)
            ('capturenow', 'yes'),
        ]

        log_payment(request, 'dibs', 'pay_start', data)

    #    if settings['CAPTURE']:
    #    data.append(('capturenow', 'yes'))

        if not settings['LIVE']:
            data.append(('test', 'yes'))

        send_data = urllib.urlencode(data)

        return HttpResponseRedirect('https://payment.architrade.com/paymentweb/start.action?' + send_data)



from cratis.features import Feature


class Compressor(Feature):
    def configure_settings(self, cls):
        self.append_apps(cls, ['compressor'])

        cls.STATICFILES_FINDERS = cls.STATICFILES_FINDERS + (
            'compressor.finders.CompressorFinder',
        )


from django.contrib.auth.signals import user_logged_in
from django.db import models

from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User


class UserProfile(models.Model):
    user = models.OneToOneField(User)
    firstname = models.CharField(max_length=255, verbose_name=_('firstname'))
    lastname = models.CharField(max_length=255, verbose_name=_('lastname'))

    def primary_address(self):
        found = None
        adresses = self.saved_adresses()
        for address in adresses:
            if address.is_main:
                found = address

        if found or len(adresses) == 0:
            return found

        return adresses[0]


    def saved_adresses(self):
        return self.adresses.filter(temporary=False)

    def __unicode__(self):
        return '%s %s <%s>' % (self.firstname, self.lastname, self.user.email)




def create_user_profile(user, **kwargs):

    try:
        user.get_profile()
    except UserProfile.DoesNotExist:
        UserProfile.objects.create(user=user)

user_logged_in.connect(create_user_profile, sender=User)




from django.forms.models import ModelForm
from cratis.app.ecommerce.orders.models import  Address

__author__ = 'Alex'

class DeliveryAddressForm(ModelForm):

    class Meta:
        model = Address
        fields = ('city', 'recipient')


import re


def require(*requirements):

    def func(object):
        def get_required(self, cls):
            parent_requirements = super(object, self).get_required(cls)

            return tuple(parent_requirements) + tuple(requirements)

        object.get_required = get_required

        return object

    return func


class Feature(object):

    def get_required(self, cls):
        return ()

    def get_required_packages(self, cls):
        return ()

    def configure_settings(self, cls):
        pass

    def configure_urls(self, cls, urls):
        pass

    def append_apps(self, cls, apps):
        for app in apps:
            if app not in cls.INSTALLED_APPS:
                cls.INSTALLED_APPS += (app,)

    def append_middleware(self, cls, classes):
        for classname in classes:
            if classname not in cls.MIDDLEWARE_CLASSES:
                cls.MIDDLEWARE_CLASSES += (classname,)

    def append_template_processor(self, cls, processors):
        for classname in processors:
            if classname not in cls.TEMPLATE_CONTEXT_PROCESSORS:
                cls.TEMPLATE_CONTEXT_PROCESSORS += (classname,)

    def report_failure(self, message_):
        print('%s -> %s' % (type(self), message_))
        exit(1)

    def require_setting(self, cls, pattern):

        pattern = re.compile('^%s$' % pattern)
        for name in dir(self):
            if pattern.match(name):
                if not hasattr(cls, name):
                    print('\n\tFeature %s require setting to be set:\n\n\t- %s\n' % (type(self), name))
                    exit(1)

                schema = getattr(self, name)

                if schema:
                    schema(getattr(cls, name))

                setattr(type(self), name, getattr(cls, name))




from cratis.features import Feature


class Content(Feature):

    def configure_settings(self, cls):
        self.append_apps(cls, ['cratis.app.content'])

        self.append_template_processor(cls, ['cratis.app.content.context.banner_context'])



"""
Views for cart manipulation.
"""

from cratis.app.ecommerce.shopping_cart.cart import Cart

from django.shortcuts import render_to_response
from django.template import RequestContext

from cratis.app.ecommerce.shopping_cart.models import DeliveryMethod
from cratis.app.ecommerce.products.models import Product


def clear_cart(request):
    """
    Clears out all content from cart
    :param request:
    :return:
    """
    cart = Cart(request)
    cart.clear()

    return render_to_response('elements/shoping_cart.html', {'shoping_cart': cart},
                              context_instance=RequestContext(request))


def set_delivery(request, product_id, method):
    """
    Set delivery method for product.
    This method is used in case of separate delivery on each product.

    :param request:
    :param product_id:
    :param method:
    :return:
    """
    product = Product.objects.get(id=product_id)
    cart = Cart(request)
    cart.set_delivery(product, DeliveryMethod.objects.get(pk=method))
    return render_to_response('elements/shoping_cart.html', {'shoping_cart': cart},
                              context_instance=RequestContext(request))


def set_product_qty(request, product_id, quantity):
    """
    Increase or decrease product quantity in cart.

    :param request:
    :param product_id:
    :param quantity:
    :return:
    """
    product = Product.objects.get(id=product_id)
    cart = Cart(request)
    cart.set_qty(product, quantity)

    return render_to_response('elements/shoping_cart.html', {'shoping_cart': cart},
                              context_instance=RequestContext(request))


def add_to_cart(request, product_id, quantity):
    """
    Add product to cart

    :param request:
    :param product_id:
    :param quantity:
    :return:
    """
    product = Product.objects.get(id=product_id)
    product.sold_count += 1
    product.save()
    cart = Cart(request)
    cart.add(product, product.price, quantity)

    response = render_to_response('elements/shoping_cart.html', {'shoping_cart': cart},
                                  context_instance=RequestContext(request))

    response.set_signed_cookie('cart_id', cart.cart.id, salt='cart_cart')

    return response


def remove_from_cart(request, product_id, quantity):
    """
    Reove product from cart.

    :param request:
    :param product_id:
    :param quantity:
    :return:
    """
    product = Product.objects.get(id=product_id)
    cart = Cart(request)
    cart.remove(product, quantity)

    return render_to_response('elements/shoping_cart.html', {'shoping_cart': cart},
                              context_instance=RequestContext(request))




from django.conf.urls import patterns, include
from cratis.features import Feature


class DjangoAllauth(Feature):

    def get_required_packages(self, cls):
        return ['django-allauth', 'django-bootstrap-form']

    def configure_settings(self, cls):

        cls.AUTHENTICATION_BACKENDS += ("allauth.account.auth_backends.AuthenticationBackend", )

        self.append_template_processor(cls, (
            "allauth.account.context_processors.account",
            "allauth.socialaccount.context_processors.socialaccount",
        ))

        self.append_apps(cls, (
            'allauth',
            'allauth.account',
            'allauth.socialaccount',

            # ... include the providers you want to enable:
            # 'allauth.socialaccount.providers.angellist',
            # 'allauth.socialaccount.providers.bitly',
            # 'allauth.socialaccount.providers.dropbox',
            'allauth.socialaccount.providers.facebook',
            # 'allauth.socialaccount.providers.feedly',
            # 'allauth.socialaccount.providers.github',
            # 'allauth.socialaccount.providers.google',
            # 'allauth.socialaccount.providers.instagram',
            # 'allauth.socialaccount.providers.linkedin',
            # 'allauth.socialaccount.providers.linkedin_oauth2',
            # 'allauth.socialaccount.providers.openid',
            # 'allauth.socialaccount.providers.persona',
            # 'allauth.socialaccount.providers.soundcloud',
            # 'allauth.socialaccount.providers.stackexchange',
            # 'allauth.socialaccount.providers.twitch',
            # 'allauth.socialaccount.providers.twitter',
            # 'allauth.socialaccount.providers.vimeo',
            # 'allauth.socialaccount.providers.vk',
            # 'allauth.socialaccount.providers.weibo',
        ))


    def configure_urls(self, cls, urls):
        urls += patterns('',
            (r'^accounts/', include('allauth.urls')),
            (r'^accounts/', include('cratis.app.profile.urls')),
        )



class DefaultProfile(Feature):

    def configure_urls(self, cls, urls):
        urls += patterns('',
            (r'^accounts/', include('cratis.app.profile.urls')),
        )


from django.conf import settings
from django.contrib.sites.models import get_current_site
from django.core.paginator import EmptyPage, PageNotAnInteger
from django.http import Http404
from django.template.response import TemplateResponse
from django.utils.translation import activate
from cratis.app.ecommerce.products.models import Product, ProductCategory

__author__ = 'alex'

from django.contrib.sitemaps import Sitemap

class ProductCategorySitemap(Sitemap):
    changefreq = "daily"
    priority = 0.5

    def items(self):
        return ProductCategory.objects.all()

    def lastmod(self, obj):
        return obj.date_updated

class ProductSitemap(Sitemap):
    changefreq = "daily"
    priority = 0.7

    def items(self):
        return Product.objects.filter(active=True, archived=False)

    def lastmod(self, obj):
        return obj.date_updated


def sitemap(request, sitemaps, section=None,
            template_name='sitemap.xml', mimetype='application/xml'):
    req_protocol = 'https' if request.is_secure() else 'http'
    req_site = get_current_site(request)

    if section is not None:
        if section not in sitemaps:
            raise Http404("No sitemap available for section: %r" % section)
        maps = [sitemaps[section]]
    else:
        maps = sitemaps.values()
    page = request.GET.get("p", 1)

    urls = []
    for site in maps:
        try:
            if callable(site):
                site = site()
            for (lang, lang_name) in settings.LANGUAGES:
                activate(lang)
                urls.extend(site.get_urls(page=page, site=req_site,
                    protocol=req_protocol))
        except EmptyPage:
            raise Http404("Page %s empty" % page)
        except PageNotAnInteger:
            raise Http404("No page '%s'" % page)

    return TemplateResponse(request, template_name, {'urlset': urls},
        content_type=mimetype)


from django.core.urlresolvers import reverse
from django.core.validators import RegexValidator
from django.db import models
from django.db.models.fields import DateTimeField
from django.utils.safestring import mark_safe

from django.utils.translation import ugettext_lazy as _
from filer.fields.image import FilerImageField
from mptt.fields import TreeForeignKey
from mptt.models import MPTTModel
from cratis.app.i18n.translate import TranslatableModelMixin, _languages
from cratis.app.i18n.templatetags.model_trans import localize


class ProductAttributeTypeTranslation(models.Model):
    lang = models.CharField(max_length=2, editable=False)
    related = models.ForeignKey('ProductAttributeType', null=True, related_name='translations')
    name = models.CharField(max_length=255, verbose_name=_('name'))


class ProductAttributeType(models.Model):
    name = models.CharField(max_length=100, verbose_name=_('name'))
    values = models.ManyToManyField('ProductAttributeValue')

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Product attribute')
        verbose_name_plural = _('Product attributes')

    @staticmethod
    def get_translation_class():
        return ProductAttributeTypeTranslation


class ProductAttributeValue(models.Model):
    type = models.ForeignKey(ProductAttributeType, null=True)
    value = models.CharField(max_length=100, verbose_name=_('value'))

    def __unicode__(self):
        return self.value


class ProductAttributeFilter(models.Model):
    name = models.CharField(max_length=100, verbose_name=_('name'))

    attributes = models.ManyToManyField('ProductAttributeType', null=True, blank=True, verbose_name=_('Attributes'))
    categories = models.ManyToManyField('ProductCategory', blank=True, null=True, verbose_name=_('Show for categories'),
                                        related_name='filters')

    order_index = models.IntegerField(verbose_name=_('Ordering index'), default=0)

    class Meta:
        ordering = ('order_index', 'name')

    def __unicode__(self):
        return self.name


class ProductAttribute(models.Model):
    product = models.ForeignKey('Product', null=True, related_name='attributes')
    type = models.ForeignKey(ProductAttributeType, null=True)
    value = models.CharField(max_length=255, verbose_name=_('value'), null=True, blank=True)

    # class Meta:
    #     ordering = ('value', )

    def __unicode__(self):
        return self.type.name


slug_validator = RegexValidator(regex='^[^\d/][^/]+$',
                                message=_('Slug should not start with digit and contain "/" character'))


class ProductCategoryTranslation(models.Model):
    lang = models.CharField(max_length=2, editable=False)
    related = models.ForeignKey('ProductCategory', null=True, related_name='translations')
    name = models.CharField(max_length=255, verbose_name=_('name'))
    description = models.TextField(blank=True, default='')
    slug = models.CharField(max_length=255, verbose_name=_('slug'), default='', blank=True, validators=[slug_validator])


class ProductCategory(TranslatableModelMixin, MPTTModel):
    date_created = DateTimeField(auto_now_add=True)
    date_updated = DateTimeField(auto_now=True)

    order_index = models.IntegerField(verbose_name=_('Ordering index'), default=0)

    name = models.CharField(max_length=255, verbose_name=_('name'))
    description = models.TextField(blank=True, default='')
    slug = models.CharField(max_length=255, verbose_name=_('slug'), default='', blank=True, validators=[slug_validator])

    image = FilerImageField(null=True, blank=True)

    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')

    is_active = models.BooleanField(default=False, verbose_name=_('Is active'))

    attribute_types = models.ManyToManyField(ProductAttributeType, verbose_name=_('Product attributes'), null=True,
                                             blank=True, related_name='categories', symmetrical=False)

    allow_filtering = models.BooleanField(default=True, verbose_name=_('Allow filter by'))

    # It is required to rebuild tree after save, when using order for mptt-tree
    def save(self, *args, **kwargs):
        super(ProductCategory, self).save(*args, **kwargs)
        ProductCategory.objects.rebuild()


    def get_absolute_url(self):
        category_slug = localize(self, 'slug')
        if category_slug != '':
            return reverse('products_in_category_by_slug', kwargs={'category_slug': category_slug})
        else:
            return reverse('products_in_category', kwargs={'category_id': self.pk})

    def recursive_category_list(self):
        if self.parent:
            all = [self]
            for cat in self.parent.recursive_category_list():
                all.append(cat)
            return all
        else:
            return [self]

        #    parents_list.short_description = 'Parent categories'

    def get_own_and_parents_filters(self):
        attributes = list(self.filters.all())
        if self.parent:
            attributes += list(self.parent.get_own_and_parents_filters())
        return attributes

    @staticmethod
    def get_translation_class():
        return ProductCategoryTranslation

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Product category')
        verbose_name_plural = _('Product categories')
        ordering = ['order_index']

    class MPTTMeta:
        order_insertion_by = ['order_index']


class ProductTranslation(models.Model):
    lang = models.CharField(max_length=2, editable=False)
    related = models.ForeignKey('Product', null=True, related_name='translations')
    title = models.CharField(max_length=255, blank=True, null=True)
    long_title = models.CharField(max_length=255, blank=True, null=True, default='')
    slug = models.CharField(max_length=255, verbose_name=_('slug'), null=True, blank=True, validators=[slug_validator])
    description = models.TextField(blank=True, null=True)
    long_description = models.TextField(blank=True, null=True)

    def __unicode__(self):

        return _languages[self.lang]

    # def title(self):
    #     return smart_str(self.title)
    #
    # def long_title(self):
    #     return smart_str(self.long_title)
    #
    # def description(self):
    #     return smart_str(self.description)
    #
    # def long_description(self):
    #     return smart_str(self.long_description)

class Product(TranslatableModelMixin, models.Model):
    date_created = DateTimeField(auto_now_add=True)
    date_updated = DateTimeField(auto_now=True)

    title = models.CharField(max_length=255)
    long_title = models.CharField(max_length=255)
    slug = models.CharField(max_length=255, verbose_name=_('slug'), default='', blank=True, validators=[slug_validator])
    description = models.TextField(blank=True)
    long_description = models.TextField(blank=True)

    price = models.DecimalField(max_digits=8, decimal_places=2)
    price_discount = models.DecimalField(max_digits=8, decimal_places=2, default=0)
    discount = models.IntegerField(default=0)

    categories = models.ManyToManyField(ProductCategory, blank=True, null=True)

    related = models.ManyToManyField('Product', blank=True, null=True)

    featured = models.BooleanField(default=False, verbose_name=_('Featured'))
    sold = models.BooleanField(default=False, verbose_name=_('Is sold'))
    active = models.BooleanField(default=False, verbose_name=_('Is active'))
    archived = models.BooleanField(default=False, verbose_name=_('Is archived'))
    sold_count = models.PositiveIntegerField(verbose_name=_('Sold count'), default=0)

    #    def title(self):
    #        return smart_str(self.title)
    #
    #    def long_title(self):
    #        return smart_str(self.long_title)
    #
    #    def description(self):
    #        return smart_str(self.description)
    #
    #    def long_description(self):
    #        return smart_str(self.long_description)

    def get_absolute_url(self):
        product_slug = localize(self, 'slug')

        if self.categories.count() < 1:
            return '/product/' + str(self.pk)

        category = self.categories.all()[0]
        category_slug = localize(category, 'slug')

        if category_slug != '' and product_slug != '':
            return reverse('product_with_cat_both_by_slug',
                           kwargs={'slug': product_slug, 'category_slug': category_slug})

        elif category_slug != '':
            return reverse('product_with_cat_cat_by_slug_prod_by_id',
                           kwargs={'pk': self.pk, 'category_slug': category_slug})

        elif product_slug != '':
            return reverse('product_with_cat_cat_by_id_prod_by_slug',
                           kwargs={'slug': product_slug, 'category_id': category.pk})

        else:
            return reverse('product_with_cat',
                           kwargs={'pk': self.pk, 'category_id': category.pk})

    def has_discount(self):
        return self.discount > 0 or self.price_discount > 0

    def discount_price(self):
        if self.price_discount > 0:
            return self.price_discount
        return self.price - self.price * self.discount / 100

    def slug_list(self):
        return ', '.join([self.slug] + [trans.slug + ' (' + trans.lang + ')' for trans in self.translations.all() if
                                        trans.slug and trans.slug != ''])

    def category_list(self):
        return ', '.join([cat.name for cat in self.categories.all()])

    category_list.short_description = 'Categories'

    @staticmethod
    def get_translation_class():
        return ProductTranslation

    def __unicode__(self):
        return self.title + ', id: ' + str(self.id)

    class Meta:
        verbose_name = _('Product')
        verbose_name_plural = _('Products')


class ProductImage(models.Model):
#    title = models.CharField(max_length=100, blank=True)
    image = FilerImageField(null=True, blank=True)
    sorting = models.PositiveIntegerField(verbose_name=_('Sorting'), default=0)
    product = models.ForeignKey(Product)

    class Meta:
        ordering = ('-sorting', )





from copy import copy
from math import floor, log10, ceil
from random import shuffle, sample
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import AuthenticationForm
from django.db.models.aggregates import Min, Max
import re
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage, InvalidPage
from django.http import HttpResponse, Http404
from django.template.context import RequestContext
from django.utils.translation import get_language, activate
from django.views.decorators.cache import cache_page
from django.views.decorators.http import require_POST
from cratis.app.i18n.utils import find_by_localized_field
from django.shortcuts import render_to_response, redirect
from django.views.generic.base import TemplateView
from django.views.generic.detail import DetailView
from django.views.generic.list import ListView
from cratis.app.i18n.translate import translate

from cratis.app.ecommerce.profile.views import WithShopingCartMixin, WithSidebarMixin
from models import Product
from cratis.app.ecommerce.orders.models import Order, DeliveryMethod
from cratis.app.ecommerce.products.models import ProductAttribute, ProductAttributeType
from django.utils.translation import ugettext as _
from cratis.app.core.common.templatetags.model_trans import localize
from cratis.app.ecommerce.products.templatetags.product_utils import product_url, category_url

from django.contrib.auth import login as auth_login, login
from django.contrib.auth.forms import PasswordResetForm
from django.contrib.auth.tokens import default_token_generator


# from haystack.query import SearchQuerySet

class SearchListView(WithShopingCartMixin, WithSidebarMixin, TemplateView):
    model = Product
    template_name = 'search.html'
    context_object_name = 'products'

    def get_filter_attributes(self, context):

        if context.has_key('cur_category'):
            filters = context['cur_category'].get_own_and_parents_filters()
        else:
            filters = []

        filtered_attributes = {}
        for filter in filters:
            for attr in filter.attributes.all():
                filtered_attributes[attr.pk] = attr

        return filtered_attributes.values()

    def transform_range(self, range, all_char='*'):
        start = range['from'] if range.has_key('from') else all_char
        end = range['to'] if range.has_key('to') else all_char
        return end, start

    def range_to_query(self, range):
        end, start = self.transform_range(range)
        return '[%s TO %s]' % (start, end)

    def update_ranges(self, ranges, data):
        end, start = self.transform_range(data)
        for idx, range in enumerate(ranges):
            r_end, r_start = self.transform_range(range)
            if r_end == end and start == r_start:
                ranges[idx]['cnt'] = data['total_count']

    def calculate_price_ranges(self):

        query = self.calculate_base_query()

        if len(query.all()) < 1:
            return []

        max_value = query.order_by('-price').best_match().price
        min_value = query.order_by('price').best_match().price

        if min_value == max_value:
            return []

        round_digits = int(floor(log10(max_value - min_value))) - 2
        min_value = int(round(min_value, -round_digits))
        max_value = int(round(max_value, -round_digits))

        range_count = 7

        step = (max_value - min_value) / range_count
        step = int(round(step, - int(floor(log10(step)))))

        if min_value != 0:
            min_value = int(round(min_value, - int(floor(log10(min_value)))))

        price_ranges = []

        for i in range(0, range_count):
            r = {}
            if i != 0:
                r['from'] = min_value + step * i
            if i != (range_count - 1):
                r['to'] = min_value + step * (i + 1)
            price_ranges.append(r)

        return price_ranges

    def prepare_filters(self, context, query):
        all_attributes = self.get_filter_attributes(context)

        query = query.facet('attributes')

        price_ranges = self.calculate_price_ranges()
        if len(price_ranges):
            query = query.query_facet('price_range', {
                "range": {
                    "field": "price",
                    "ranges": price_ranges
                }
            })

        facets = {}
        query_facet_counts = query.facet_counts()

        for idx, value in context['selection'].items():
            if idx == 'price':
                query = query.narrow(
                    u'price:%s' % self.range_to_query(price_ranges[int(context['selection']['price'])]))
            else:
                query = query.narrow(u'attributes:"%s"' % (idx + '_' + value))

        filters = []

        if 'fields' in query_facet_counts:
            for facet in query_facet_counts['fields']['attributes']:
                (value, cnt) = facet
                match = re.match('^(\d+)_(.+)$', value)
                if match:
                    i = int(match.group(1))
                    if not i in facets:
                        facets[i] = {}
                    facets[i][match.group(2)] = cnt

            for attribute in all_attributes:
                if facets.has_key(attribute.pk):
                    filters.append({
                        'attribute': localize(attribute, "name"),
                        'id': attribute.pk,
                        'values': sorted([
                            {
                                'id': val,
                                'selected': True if context['selection'].has_key(u'%s' % attribute.pk) and
                                                    context['selection'][u'%s' % attribute.pk] == val else False,
                                'title': val,
                                'cnt': cnt
                            } for val, cnt in facets[attribute.pk].items()
                        ], key=lambda item: item['title'])
                    })

            if 'price_range' in query_facet_counts['queries']:
            #                for range in query_facet_counts['queries']['price_range']['ranges']:


                for range in query_facet_counts['queries']['price_range']['ranges']:
                    self.update_ranges(price_ranges, range)

                for idx, range in enumerate(price_ranges):
                    range['id'] = idx
                    end, start = self.transform_range(range, '...')
                    range['title'] = '%s - %s' % (start, end)
                    range['selected'] = False

                if context['selection'].has_key('price'):
                    selected_price = int(context['selection']['price'])
                    price_ranges[selected_price]['selected'] = True

                filters.append({
                    'attribute': _('Price'),
                    'id': 'price',
                    'values': price_ranges
                })
        context['filters'] = filters
        return query

    def calculate_base_query(self):
        query = SearchQuerySet()
        if self.request.GET.has_key('q'):
            query = query.filter(content=self.request.GET['q'])
        if 'category_id' in self.kwargs:
            query = query.filter(categories=self.kwargs['category_id'])
        return query

    def get_context_data(self, **kwargs):

        context = super(SearchListView, self).get_context_data(**kwargs)

        query = self.calculate_base_query()

        if self.request.GET.has_key('q'):
            context['q'] = self.request.GET['q']

        context['query'] = query

        if self.request.GET.has_key('sort'):
            sort = self.request.GET['sort']
            if sort == 'discount':
                query = query.order_by('-discount')
            elif sort == 'high':
                query = query.order_by('-price')
            elif sort == 'bestseller':
                query = query.order_by('-sold')
            elif sort == 'offers':
                query = query.filter(featured=True).order_by('-sold')
            else:
                sort = 'low'
                query = query.order_by('price')
        else:
            sort = 'low'
            query = query.order_by('price')
        context['sorting'] = sort

        selection = {}
        for key, val in self.request.GET.items():
            if key[0] == '_':
                key = key[1:]
                selection[key] = val
        context['selection'] = selection

        query = self.prepare_filters(context, query)

        if 'cur_category' in context:
        #            context['filters'] = []

            lang_urls = {}
            cur_lang = get_language()
            for code, name in LANGUAGES:
                activate(code)
                lang_urls[code] = category_url(context['cur_category'])
            activate(cur_lang)

            context['override_lang_urls'] = lang_urls

        results = query.all()

        paginator = Paginator(results, 10)

        page = 1
        if self.request.GET.has_key('page'):
            page = self.request.GET['page']

        try:
            page = paginator.page(page)
        except InvalidPage:
            raise Http404("No such page!")

        context['page'] = page

        context['compare_products'] = generate_compare_data(self.request)

        return context


class MainPageListView(WithShopingCartMixin, WithSidebarMixin, ListView):
    model = Product
    template_name = 'profile.html'
    context_object_name = 'products'

    def get_context_data(self, **kwargs):

        context = super(MainPageListView, self).get_context_data(**kwargs)

        query_set = Product.objects.filter(featured=True, active=True)

        if settings.SHOP_PRODUCTS_HIDE_SOLD:
            query_set = query_set.filter(sold=False)

        featured = query_set[:30]
        if len(featured) > 3:
            context['featured'] = sample(featured, 3)
        else:
            context['featured'] = query_set

        query_set = Product.objects.filter(active=True).order_by('-sold_count')

        if settings.SHOP_PRODUCTS_HIDE_SOLD:
            query_set = query_set.filter(sold=False)

        bought = query_set[:30]
        if len(bought) > 3:
            context['bought'] = sample(bought, 3)
        else:
            context['bought'] = query_set

        context['compare_products'] = generate_compare_data(self.request)

        return context

    def get_queryset(self):
        q = Product.objects.all()

        if settings.SHOP_PRODUCTS_HIDE_SOLD:
            q = q.filter(sold=False)

        paginator = Paginator(q, 10)
        page = self.request.GET.get('page')

        try:
            products = paginator.page(page)
        except PageNotAnInteger:
            products = paginator.page(1)
        except EmptyPage:
            products = paginator.page(paginator.num_pages)

        return products


class ProductView(WithShopingCartMixin, WithSidebarMixin, DetailView):
    model = Product
    template_name = 'product.html'
    context_object_name = 'product'

    def get_context_data(self, **kwargs):
        context = super(ProductView, self).get_context_data(**kwargs)

        lang_urls = {}
        cur_lang = get_language()
        for code, name in settings.LANGUAGES:
            activate(code)
            lang_urls[code] = product_url(context['product'],
                                          context['cur_category'] if 'cur_category' in context else None)
        activate(cur_lang)

        context['override_lang_urls'] = lang_urls

        return context

    def get_object(self, queryset=None):

        result = None
        if 'slug' in self.kwargs:
            result = find_by_localized_field(Product, 'slug', self.kwargs['slug'])
            if not result:
                raise Http404(_(u"No Products found matching the query"))
        else:
            result = super(ProductView, self).get_object(queryset)

        if not result or result.archived or not result.active:
            raise Http404(_(u"No Products found matching the query"))

        return result



def generate_compare_data(request):
    compare = request.session.get('compare', [])
    products = Product.objects.filter(id__in=compare)
    return products


def render_compare(request):
    return render_to_response('elements/compare.html', {'compare_products': generate_compare_data(request)},
                              context_instance=RequestContext(request))


def add_to_compare(request, product_id):
    compare = request.session.get('compare', [])
    compare.append(product_id)

    request.session['compare'] = compare
    return render_compare(request)


def clear_compare(request):
    request.session['compare'] = []
    return render_compare(request)


@require_POST
@login_required
def reject_order(request, order_id):
    try:
        order = Order.objects.get(pk=order_id, user=request.user.get_profile())
        if order.status == Order.ORDER_STATUS_CONFIRMED:
            order.status = Order.ORDER_STATUS_REJECTED
            order.save()
        return HttpResponse(json.dumps({'ok': True}), mimetype='application/json')
    except Exception, e:
        return HttpResponse(json.dumps({'ok': False, 'error': e.message}), mimetype='application/json')


@require_POST
@login_required
def repeat_order(request, order_id):
    try:
        from cratis.app.ecommerce.shopping_cart import Cart
        order = Order.objects.get(pk=order_id, user=request.user.get_profile())
        cart = Cart(request)

        for item in order.cart.items.all():
            cart.add(item.product, item.product.price, item.quantity)

        return render_to_response('elements/shoping_cart.html', {'shoping_cart': cart},
                                  context_instance=RequestContext(request))
    except Exception, e:
        return HttpResponse(json.dumps({'ok': False, 'error': e.message}), mimetype='application/json')

@require_POST
def trans(request):
    if request.POST['reverse'] == 'true':
        result = translate(request.POST['message'], request.POST['lang'], MAIN_LANGUAGE)
    else:
        result = translate(request.POST['message'], MAIN_LANGUAGE, request.POST['lang'])

    try:
        return HttpResponse(json.dumps({'ok': True, 'result': result}), mimetype='application/json')
    except Exception, e:
        return HttpResponse(json.dumps({'ok': False, 'error': e.message}), mimetype='application/json')


def attr_type(request, type):
    values = [attr.value for attr in ProductAttribute.objects.filter(type=type) if
              attr.value is not None and len(attr.value) > 0]
    values = list(set(values)) # remove duplicates
    return HttpResponse(json.dumps(values), mimetype='application/json')

@cache_page(60 * 10)
def all_products_xml(request):
    return render_to_response('all_products.html', {'products': Product.objects.filter(active=True)},
                              mimetype='application/xml')




from django.utils.encoding import smart_str
from cratis.app.ecommerce.products.models import Product

__author__ = 'alex'
from haystack import indexes


class ProductIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.NgramField(document=True, use_template=True)
    categories = indexes.MultiValueField(faceted=True)
    attributes = indexes.FacetMultiValueField()
    price = indexes.IntegerField(faceted=True)
    sold = indexes.IntegerField()
    featured = indexes.BooleanField()
    discount = indexes.IntegerField(model_attr='discount')

    def get_model(self):
        return Product

    def should_update(self, instance, **kwargs):
        if not instance.active:
            self.remove_object(instance)
            return False
        return super(ProductIndex, self).should_update(instance, **kwargs)

    def index_queryset(self, **kwargs):
        return Product.objects.filter(active=True, archived=False)

    def prepare_attributes(self, obj):
        return [str(attr.type.pk) + '_' + smart_str(attr.value) for attr in obj.attributes.all()]

    def prepare_featured(self, obj):
        return True if obj.featured else False

    def prepare_sold(self, obj):
        return obj.sold_count

    def prepare_price(self, obj):
        return int(obj.price)

    def prepare_categories(self, obj):
        ids = []

        for category in obj.categories.all():
            for id in [cat.id for cat in category.recursive_category_list()]:
                ids.append(id)
        return set(ids)




# -*- coding: utf-8 -*-

from datetime import datetime
from decimal import Decimal, ROUND_DOWN, ROUND_UP
import hashlib
from django.core.validators import validate_email
from django.db import models
from django.db.models.deletion import PROTECT, SET_NULL
from django.db.models.fields import DateTimeField
from cratis.app.ecommerce.payment.models import PaymentMethod
from cratis.app.ecommerce.shopping_cart.models import Cart, Item, DeliveryMethod
from django.utils.translation import ugettext_lazy as _
from django.utils import unittest
from cratis.app.ecommerce.profile.models import UserProfile

from paypal.pro.signals import payment_was_successful

__author__ = 'alex'

def show_me_the_money(sender, **kwargs):
    item = sender
    order_id = item['inv']

    order = Order.objects.get(pk=order_id)
    order.status = Order.ORDER_STATUS_PAID
    order.save()

payment_was_successful.connect(show_me_the_money)

class Address(models.Model):
    street_address = models.CharField(max_length=250, null=False, default="", blank=False)
    city = models.CharField(max_length=250, null=False, default="", blank=False)
    post_index = models.CharField(max_length=250, null=False, default="", blank=False)
    country = models.CharField(max_length=250, null=False, default="", blank=False)
    recipient = models.CharField(max_length=250, null=False, default="", blank=True)
    firstname = models.CharField(max_length=250, null=False, default="", blank=False)
    lastname = models.CharField(max_length=250, null=False, default="", blank=False)
    email = models.CharField(max_length=250, null=False, default="", blank=False, validators=[validate_email])
    phone = models.CharField(max_length=250, null=False, default="", blank=False)

    temporary = models.BooleanField(default=False, blank=True)

    is_main = models.BooleanField(default=False, blank=True, verbose_name=_('Default address'))

    profile = models.ForeignKey(UserProfile, null=True, blank=True, default=None,  related_name='adresses')

    def is_ready(self):
        return self.city and \
               self.street_address and \
               self.post_index and \
               self.lastname and \
               self.firstname and \
               self.email and \
               self.phone

    def __unicode__(self):
        return ', '.join([str(self.country), self.city, str(self.post_index), self.street_address, self.recipient])


class Order(models.Model):
    date_created = DateTimeField(auto_now_add=True)
    date_updated = DateTimeField(auto_now=True)

    cart = models.ForeignKey(Cart, related_name='_cart', null=True, on_delete=PROTECT)
    user = models.ForeignKey(UserProfile, related_name='orders', null=True, blank=True, on_delete=SET_NULL)

    email = models.CharField(max_length=150, null=True)
    phone = models.CharField(max_length=40, null=True)

    invoiceData = models.TextField(blank=True, default='')
    is_sent = models.BooleanField(default=True, verbose_name=_('Invoice has been delivered'))

    ip_address = models.CharField(max_length=40, null=True)

    #delivery method

    is_test = models.BooleanField(default=True, verbose_name=_('Is test payment'))

    comment = models.TextField(blank=True, null=True)

    # status

    ORDER_STATUS_NEW = 'new'
    ORDER_STATUS_CONFIRMED = 'confirmed'
    ORDER_STATUS_PAID = 'shipping'
    ORDER_STATUS_COMPLETED = 'completed'
    ORDER_STATUS_RETURNED = 'returned'
    ORDER_STATUS_REJECTED= 'rejected'

    # status
    ORDER_STATUSES = (
        (ORDER_STATUS_NEW, _('New order')),
        (ORDER_STATUS_CONFIRMED, _('Confirmed, waiting for payment')),
        (ORDER_STATUS_PAID, _('Paid, shipping to customer')),
        (ORDER_STATUS_COMPLETED, _('Paid and delivered')),
        (ORDER_STATUS_RETURNED, _('Returned')),
        (ORDER_STATUS_REJECTED, _('Rejected by user')),
    )

    status = models.CharField(max_length=20, choices=ORDER_STATUSES, default=ORDER_STATUS_NEW)

    payment_status = models.CharField(max_length=10, null=True)
    payment_response = models.TextField(blank=True, null=True)

    delivery_method = models.ForeignKey(DeliveryMethod, null=True, on_delete=PROTECT)
    payment_method = models.ForeignKey(PaymentMethod, null=True, on_delete=PROTECT)

    address = models.ForeignKey(Address, null=True, on_delete=SET_NULL)

    smartpost_box = models.IntegerField(null=True)
    smartpost_box_name = models.TextField(blank=True, null=True)

    def full_price(self):
        return self.get_detalisation()['full_total']

    def item_names(self):
        return ', '.join([item.product.title for item in Item.objects.filter(cart=self.cart)])

    def get_linkhash(self):
        return hashlib.sha224(str(self.date_created) + 'fjdkjl' + str(self.id)).hexdigest()

    def fill_items(self):
        if self.cart:
            self.cart.order = self
            for item in self.cart.items.all():
                item.order = self
                item.save()

    def documents(self):
        return 'Invoice: <a href="%s" target="_blank"><img style="margin-bottom: -8px;" src="/assets/img/pdf_icon.gif" alt=""></a>' % self.invoice_pdf_url() + \
        ' Shipping list: <a href="%s" target="_blank"><img style="margin-bottom: -8px;" src="/assets/img/pdf_icon.gif" alt=""></a>' % self.shipping_list_pdf_url()
    documents.allow_tags = True

    def shipping_list_pdf_url_href(self):
        return '<a href="%s" target="_blank"><img style="margin-bottom: -8px;" src="/assets/img/pdf_icon.gif" alt=""></a>' % self.shipping_list_pdf_url()
    shipping_list_pdf_url_href.allow_tags = True

    def invoice_pdf_url_href(self):
        return '<a href="%s" target="_blank"><img style="margin-bottom: -8px;" src="/assets/img/pdf_icon.gif" alt=""></a>' % self.invoice_pdf_url()
    invoice_pdf_url_href.allow_tags = True

    def invoice_pdf_url(self):
        return 'https://s3.amazonaws.com/maksa-shops-invoice/invoice-%s-%s.pdf' % (
            hashlib.sha256(str(self.pk) + "hoho12").hexdigest(),
            hashlib.sha256(str(self.pk) + "hoho21").hexdigest(),
        )
    def shipping_list_pdf_url(self):
        return 'https://s3.amazonaws.com/maksa-shops-invoice/shipping-list-%s-%s.pdf' % (
            hashlib.sha256(str(self.pk) + "hoho12").hexdigest(),
            hashlib.sha256(str(self.pk) + "hoho21").hexdigest(),
        )


    def is_paid(self):
        return self.status == self.ORDER_STATUS_PAID or self.status == self.ORDER_STATUS_COMPLETED

    def can_pay(self):
        return self.status == self.ORDER_STATUS_NEW or self.status == self.ORDER_STATUS_CONFIRMED

    def mark_paid(self):
        self.status = self.ORDER_STATUS_PAID
        self.save()

    def mark_comleted(self):
        self.status = self.ORDER_STATUS_COMPLETED
        self.save()

    def mark_confirmed(self):
        self.fill_items()
        if self.status == self.ORDER_STATUS_NEW:
            self.status = self.ORDER_STATUS_CONFIRMED
        self.date_created = datetime.now()
        self.save()

#        if SHOP_COUNT_PRODUCTS:
#            for item in self.cart.items.all(): # for products that are only in 1 exemplar
#                item.product.sold = True
#                item.product.save()

    def is_empty(self):
        return len(self.cart.items.all) > 0

    def get_detalisation(self):
        rows = []
        cart_total_price = self.cart.total_price
        discount_ = 0

        rows.append({
            'title': u'Сумаа заказа',
            'price': cart_total_price,
        })
        if self.delivery_method:

            rows.append({
                'title': _('Delivery') + ' - ' + self.delivery_method.name,
                'price': self.delivery_method.delivery_price,
            })
        else:
            rows.append({
                'title': _('no delivery'),
                'price': 0,
            })
        #if self.payment_method and self.payment_method.discount > 0:
        #    discount_ = (cart_total_price * self.payment_method.discount * -1).quantize(Decimal('.01'), rounding=ROUND_UP)
        #    rows.append({
        #        'title': _(u'Discount') + ' - '
        #                 + str((self.payment_method.discount * 100).quantize(Decimal('1.'))) + '%' ,
        #        'price': discount_,
        #    })
#        else:
#            rows.append({
#                'title': _('No discounts'),
#                'price': 0,
#                })

        full_total = Decimal('0.0')
        for row in rows:
            full_total += row['price']

        return {
            'full_total': full_total.quantize(Decimal('.01'), rounding=ROUND_DOWN),
            'rows': rows
        }
    details = property(get_detalisation)

    def __unicode__(self):
        return "Order %s" % self.id

    class Meta:
        verbose_name = _('Order')
        verbose_name_plural = _('Orders')

class OrderTestCase(unittest.TestCase):

    def test_create_default_order(self):

        method1 = DeliveryMethod(name='delivery-pickup', delivery_price=Decimal('1.30'))
        method1.save()

        payment1 = PaymentMethod(name='payment-pangalink', discount=0)
        payment1.save()

        cart = Cart()
        cart.creation_date = datetime.now()
        cart.save()

        order = Order()
        order.status = Order.ORDER_STATUS_NEW
        order.cart = cart
        order.delivery_method = DeliveryMethod.objects.get(name='delivery-pickup')
        order.payment_method = PaymentMethod.objects.get(name='payment-pangalink')
        order.save()

        self.assertIsNotNone(Order.objects.get(pk=order.id).delivery_method)
        self.assertIsNotNone(Order.objects.get(pk=order.id).payment_method)

    def test_details_with_empty_order(self):
        order = Order()
        order.cart = Cart()

        details = order.details

        self.assertEqual(0, details['full_total'])
        self.assertEqual(3, len(details['rows'])) # Order record

    def test_details_with_delivery(self):

        order = Order()
        order.delivery_method = DeliveryMethod()
        order.delivery_method.name = 'mock'
        order.delivery_method.delivery_price = Decimal('2.60')
        order.cart = Cart()

        details = order.details

        self.assertEqual(Decimal('2.60'), details['full_total'])
        self.assertEqual(3, len(details['rows'])) # Order record


    def test_details_with_payment_discount(self):

        order = Order()
        order.delivery_method = DeliveryMethod()
        order.delivery_method.name = 'mock'
        order.delivery_method.delivery_price = Decimal('2.60')

        order.cart = Cart()
        order.cart.creation_date = datetime.now()
        order.cart.save()

        item1 = Item()
        item1.quantity = 2
        item1.unit_price = Decimal('1.10')
        item1.cart = order.cart
        item1.save()

        pm = PaymentMethod()
        pm.name = 'mock'
        pm.discount = Decimal('0.03')

        order.payment_method = pm

        details = order.details

        self.assertEqual(Decimal('4.73'), details['full_total'])
        self.assertEqual(3, len(details['rows'])) # Order record




import datetime
from decimal import Decimal
import models
from cratis.app.core.settings import SHOP_DELIVERY_PER_PRODUCT
from cratis.app.core.common.templatetags.model_trans import localize

CART_ID = 'CART-ID'


class ItemAlreadyExists(Exception):
    pass


class ItemDoesNotExist(Exception):
    pass


class Cart:
    def __init__(self, request, cart=None):

        if cart:
            self.cart = cart
        else:

            cart_id = request.session.get(CART_ID)

            if not cart_id:
                cart_id = request.get_signed_cookie('cart_id', salt='cart_cart', default=None)

            if cart_id:
                try:
                    cart = models.Cart.objects.get(id=cart_id, checked_out=False)
                except models.Cart.DoesNotExist:
                    cart = self.new(request)
            else:
                cart = self.new(request)
            self.cart = cart

    def __len__(self):
        return len(self.cart.items.all())

    def __iter__(self):
        for item in self.cart.items.all():
            yield item

    def new(self, request):

        cart = models.Cart(creation_date=datetime.datetime.now())
        cart.save()
        request.session[CART_ID] = cart.id
        return cart

    def in_cart(self, item_id):
        if self.is_empty():
            return False

        for item in self.cart.items.all():
            if item.object_id == item_id:
                return True

        return False

    #    def is_locked(self):
    #        return self.cart.order.status != orders.models.Order.ORDER_STATUS_NEW

    def is_empty(self):
        return len(self.cart.items.all()) < 1

    def info(self):
        items = self.cart.items.all()

        total = 0
        sum = Decimal(0)
        for item in items:
            total += item.quantity
            sum += item.total_price

        return {
            'items': [
                {
                    'id': item.product.id,
                    'title': localize(item.product, 'title'),
                    'qty': item.quantity,
                    'delivery_method': item.delivery_method,
                    'price_total': "%01.2f" % item.total_price,
                    'price_unit': "%01.2f" % item.unit_price
                } for item in items],
            'size': len(items),
            'total_elements': total,
            'without_tax': self.cart.without_tax,
            'tax': self.cart.tax,
            'tax_formated': "%01.2f" % (float(sum) * 0.25),
            'total_price_raw': sum,
            'total_price': "%01.2f" % sum
        }


    def set_delivery(self, product, delivery_method):
        try:
            item = models.Item.objects.get(
                cart=self.cart,
                product=product,
            )
        except models.Item.DoesNotExist:
            raise ItemDoesNotExist
        else:
            item.delivery_method = delivery_method
            item.save()

    def set_qty(self, product, quantity):
        try:
            item = models.Item.objects.get(
                cart=self.cart,
                product=product,
            )
        except models.Item.DoesNotExist:
            raise ItemDoesNotExist
        else:
            item.quantity = long(quantity)
            item.save()

    def add(self, product, unit_price, quantity=1):
        try:
            item = models.Item.objects.get(
                cart=self.cart,
                product=product,
            )
        except models.Item.DoesNotExist:
            item = models.Item()
            item.cart = self.cart
            item.product = product

            # select best price
            if product.price_discount and 0 < product.price_discount < product.price:
                item.unit_price = product.price_discount  # unit_price
            else:
                item.unit_price = product.price  # unit_price

            item.quantity = quantity

            if SHOP_DELIVERY_PER_PRODUCT:
                methods = models.DeliveryMethod.objects.filter(enabled=True)
                if len(methods) > 1:
                    item.delivery_method = methods[0]

            item.save()
        else:
            item.quantity += long(quantity)
            item.save()

    def remove(self, product, quantity=0):

        quantity = long(quantity)
        try:
            item = models.Item.objects.get(
                cart=self.cart,
                product=product,
            )
        except models.Item.DoesNotExist:
            raise ItemDoesNotExist
        else:
            print quantity
            if quantity:
                item.quantity -= long(quantity)
                if item.quantity < 1:
                    item.delete()
                else:
                    item.save()
            else:
                item.delete()

    def update(self, product, quantity, unit_price=None):
        try:
            item = models.Item.objects.get(
                cart=self.cart,
                product=product,
            )
        except models.Item.DoesNotExist:
            raise ItemDoesNotExist

    def clear(self):
        for item in self.cart.items.all():
            item.delete()






import os
from cratis.features import Feature


class AdminArea(Feature):

    def __init__(self, prefix=r'^admin/'):
        super(AdminArea, self).__init__()
        self.prefix = prefix

    def configure_settings(self, cls):
        cls.INSTALLED_APPS += ('django.contrib.admin',)

    def configure_urls(self, cls, urls):

        from django.conf.urls import patterns, url, include
        from django.contrib import admin

        admin.autodiscover()

        urls += patterns('',
            url(self.prefix, include(admin.site.urls)),
        )


class AdminThemeSuit(Feature):

    def __init__(self, title='My site'):
        self.title = title

    def get_required_packages(self, cls):
        return 'django-suit',


    def configure_settings(self, cls):

        cls.INSTALLED_APPS += ('suit',)

        if not 'django.core.context_processors.request' in cls.TEMPLATE_CONTEXT_PROCESSORS:
            cls.TEMPLATE_CONTEXT_PROCESSORS += ('django.core.context_processors.request',)

        cls.SUIT_CONFIG = {
            'ADMIN_NAME': self.title
        }

        cls.TEMPLATE_DIRS += (os.path.dirname(os.path.dirname(__file__)) + '/templates/suit-feature',)



import os
import pkg_resources
from cratis.features import Feature


class Theme(Feature):
    def __init__(self, name):
        super(Theme, self).__init__()

        self.theme_name = name

    def load_theme(self, cls):

        from cratis.utils.themes import load_theme
        themes_dir = cls.BASE_DIR + os.sep + 'themes'

        themes = {}
        for entry_point in pkg_resources.iter_entry_points('cratis.themes'):
            themes[entry_point.name] = entry_point.load()()  # laod and execute right a way

        theme_config = load_theme({'dir': themes_dir, 'themes': themes}, self.theme_name)


        return theme_config



    def configure_settings(self, cls):
        super(Theme, self).configure_settings(cls)

        from cratis.utils.themes import ThemeLoadException

        try:
            theme_config = self.load_theme(cls)
            cls.THEME_NAME = self.theme_name
            cls.CMS_TEMPLATES += tuple([(path, name) for name, path in theme_config['cms_templates'].items()])

            cls.STATICFILES_DIRS += tuple(theme_config['asset_dirs'])
            cls.TEMPLATE_DIRS += tuple(theme_config['template_dirs'])

        except ThemeLoadException as e:
            self.report_failure('Failed to load theme: %s. Reason: %s' % (self.theme_name, e.message))


class TwitterBootstrap(Feature):

    def configure_settings(self, cls):
        self.append_apps(cls, ('bootstrapform',))


from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from cms.models.pluginmodel import CMSPlugin
from django.utils.translation import ugettext_lazy as _

class SliderPlugin(CMSPluginBase):
    model = CMSPlugin
    render_template = "content/slider.html"

# plugin_pool.register_plugin(SliderPlugin)


from django.contrib import admin
from cratis.app.core.common.translate import TranslatableAdmin, InlineTranslations
from cratis.app.ecommerce.orders.models import Order, Address
from cratis.app.ecommerce.shopping_cart.models import  Item as CartItem, DeliveryMethod, DeliveryMethodTranslation
from cratis.app.ecommerce.orders.views import add_invoice_delivery_task

class InlineAddress(admin.StackedInline):
    model = Address
    extra = 0
#    fields = ('product', 'quantity', 'unit_price')
#    readonly_fields = ('quantity', 'unit_price', 'product',)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

class InlineOrderItems(admin.TabularInline):
    model = CartItem
    extra = 0
    fields = ('product', 'quantity', 'unit_price')
    readonly_fields = ('quantity', 'unit_price', 'product',)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


def fix_translations(modeladmin, request, queryset):
    for obj in queryset:
        modeladmin.fix_translations(obj)
fix_translations.short_description = "Fix empty translations"

def generate_invoice(modeladmin, request, queryset):
    for obj in queryset:
        add_invoice_delivery_task(obj)

generate_invoice.short_description = "Generate broken invoice"

class OrderAdmin(admin.ModelAdmin):

    list_display = ('date_created', 'email', 'full_price', 'status', 'payment_method', 'documents', 'ip_address')
    inlines = [InlineOrderItems]
    readonly_fields = ('delivery_method', 'address', 'payment_method', 'ip_address',)
    exclude = ('smartpost_box','payment_status', 'phone', 'payment_response', 'smartpost_box_name')
    list_filter = ('status', 'ip_address',)
    ordering = ('-date_created',)
    actions = [generate_invoice]



admin.site.register(Order, OrderAdmin)



class InlineDeliveryMethodTranslation(InlineTranslations):
    model = DeliveryMethodTranslation

class DeliveryMethodAdmin(TranslatableAdmin):
    inlines = [InlineDeliveryMethodTranslation]
    init_translations = ['name']
    translation_class = DeliveryMethodTranslation
    list_display = ('name', 'method', 'enabled', 'translation_list')
    actions = [fix_translations]


admin.site.register(DeliveryMethod, DeliveryMethodAdmin)






import re
from voluptuous import Schema, Invalid

__author__ = 'alex'


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


class NotImplemented(Exception):
    pass


class Payment(object):

    actions_template = None

    def __init__(self, config, keys):
        self.keys = keys
        self.apply_config(config)

        super(Payment, self).__init__()

    def actions_template_name(self):
        return self.actions_template

    def actions_context(self):
        return {}

    def valid_key(self):

        def key_validator(val):
            if len(val) and val[0] == '$':
                val = val[1:]
                if not val in self.keys:
                    raise Invalid('No such key %s' % val)
                return self.keys[val]
            else:
                raise Invalid('Key field must contain reference to a key: %s' % val)

        return key_validator

    def schema(self):
        return Schema({})


    def apply_config(self, config):
        if config:
            schema = self.schema()
            self.config = schema(config)

    def require_payment(self):
        return False




def log_payment(request, module, operation, data=None):
    from cratis.app.ecommerce.payment.models import PaymentLog
    rec = PaymentLog()
    rec.module = module
    rec.operation = operation
    rec.url = request.path

    if not request.user.is_anonymous():
        rec.user_id = request.user.pk

    rec.user_ip = get_client_ip(request)
    rec.request_get = str(request.GET)
    rec.request_post = str(request.POST)

    if data:
        rec.data = str(data)

    rec.save()


from django.conf.urls.defaults import patterns, include, url

# Uncomment the next two lines to enable the admin:
from cratis.app.i18n.utils import localize_url as _
from cratis.app.ecommerce.profile.views import Profile, ProfileAddressList, ProfileAddressNew, ProfileOrderList, ProfileOrderDetails, ProfileAddressEdit, ProfileAddressRemove, ProfileEdit
from cratis.app.ecommerce.products.sitemap import ProductCategorySitemap, ProductSitemap
from cratis.app.ecommerce.products.views import MainPageListView, ProductView, SearchListView, all_products_xml

sitemaps = {
    'categories': ProductCategorySitemap(),
    'products': ProductSitemap(),
    }

urlpatterns = patterns('',

    url(_(r'^$'), MainPageListView.as_view(), name='main_page'),

    url(r'^sitemap\.xml$', 'cratis.app.ecommerce.products.sitemap.sitemap', {'sitemaps': sitemaps}),
    url(r'^afesf322fs32fefwkj32iiwefw/all_products\.xml$', all_products_xml),

    url(_(r'^product/all/$'), SearchListView.as_view(), name='all_products'),
    url(_(r'^product/$'), SearchListView.as_view()),

    url(_(r'^product/(?P<category_id>\d+)$'),
        SearchListView.as_view(),
        name='products_in_category'
    ),

    url(_(r'^product/all/(?P<pk>\d+)$'),
        ProductView.as_view(),
        name='product'
    ),

    url(_(r'^product/(?P<category_id>\d+)/(?P<pk>\d+)$'),
        ProductView.as_view(),
        name='product_with_cat'
    ),

    url(_(r'^product/all/(?P<slug>[^/]+)$'),
        ProductView.as_view(),
        name='product_by_slug'
    ),

    url(_(r'^product/(?P<category_slug>[^/]+)$'),
        SearchListView.as_view(),
        name='products_in_category_by_slug'
    ),

    url(_(r'^product/(?P<category_slug>[^/]+)/(?P<pk>\d+)$'),
        ProductView.as_view(),
        name='product_with_cat_cat_by_slug_prod_by_id'
    ),

    url(_(r'^product/(?P<category_id>\d+)/(?P<slug>[^/]+)$'),
        ProductView.as_view(),
        name='product_with_cat_cat_by_id_prod_by_slug'
    ),

    url(_(r'^product/(?P<category_slug>[^/]+)/(?P<slug>[^/]+)$'),
        ProductView.as_view(),
        name='product_with_cat_both_by_slug'
    ),


    url(_(r'^profile/$'),
        Profile.as_view(),
        name='profile'
    ),

    url(_(r'^profile/edit$'),
        ProfileEdit.as_view(),
        name='profile_edit'
    ),

    url(_(r'^profile/address/$'),
        ProfileAddressList.as_view(),
        name='profile_address'
    ),
    url(_(r'^profile/address/make_default/(?P<id>\d+)$'),
        'cratis.app.ecommerce.profile.views.make_address_default',
        name='profile_address_make_default'
    ),
    url(_(r'^profile/address/update_address/(?P<id>\d+)$'),
        'cratis.app.ecommerce.profile.views.update_address',
        name='profile_update_address'
    ),
    url(_(r'^profile/orders/$'),
        ProfileOrderList.as_view(),
        name='profile_orders'
    ),
    url(_(r'^profile/orders/detail/(?P<pk>\d+)$'),
        ProfileOrderDetails.as_view(),
        name='profile_order_details'
    ),

    url(_(r'^profile/address/new$'),
        ProfileAddressNew.as_view(),
        name='profile_address_new'
    ),

    url(_(r'^profile/address/edit/(?P<pk>\d+)$'),
        ProfileAddressEdit.as_view(),
        name='profile_address_edit'
    ),


    url(_(r'^profile/address/remove/(?P<pk>\d+)$'),
        ProfileAddressRemove.as_view(),
        name='profile_address_remove'
    ),

    url(r'^compare/add/(?P<product_id>\d+)$', 'cratis.app.ecommerce.products.views.add_to_compare', name='compare_add'),
    url(r'^compare/clear$', 'cratis.app.ecommerce.products.views.clear_compare', name='compare_clear'),
    url(r'^compare/render$', 'cratis.app.ecommerce.products.views.render_compare', name='compare_render'),

    url(r'^order/reject/(?P<order_id>\d+)$', 'cratis.app.ecommerce.products.views.reject_order', name='reject_order'),
    url(r'^order/repeat/(?P<order_id>\d+)$', 'cratis.app.ecommerce.products.views.repeat_order', name='repeat_order'),

    url(r'^trans$', 'cratis.app.ecommerce.products.views.trans'),
    url(r'^type-ac/(?P<type>\d+)$', 'cratis.app.ecommerce.products.views.attr_type'),

    url(_(r'^ajax-auth/login$'), 'cratis.app.ecommerce.products.views.ajax_login', name='ajax_login'),
    url(_(r'^ajax-auth/register$'), 'cratis.app.ecommerce.products.views.ajax_register', name='ajax_register'),
    url(_(r'^ajax-auth/password_restore$'), 'cratis.app.ecommerce.products.views.ajax_password_restore', name='ajax_password_restore'),

)



from django.conf.urls.defaults import patterns, url
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt
from cratis.app.ecommerce.payment.kardikeskus.views import KardikeskusStart, KardikeskusCallback


urlpatterns = patterns('cratis.app.ecommerce.payment.kardikeskus.views',
    url(r'^(?P<method>[^\\]+)/start', never_cache(KardikeskusStart.as_view()), name='kardikeskus_payment_start'),
    url(r'^(?P<method>[^\\]+)/callback', csrf_exempt(never_cache(KardikeskusCallback.as_view())), name='kardikeskus_payment_callback'),
)






from django.conf.urls.defaults import patterns, url
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt
from cratis.app.ecommerce.payment.voucher.views import VoucherStart


urlpatterns = patterns('cratis.app.ecommerce.payment.voucher.views',
    url(r'^(?P<method>[^\\]+)/pay', never_cache(VoucherStart.as_view()), name='voucher_payment_start')
)






#!/usr/bin/python
from django.conf import settings

from django.contrib import admin

#from apiclient.discovery import build
from django.contrib.admin.options import InlineModelAdmin

class TranslatableModelMixin(object):

    def translation_list(self):

        if not hasattr(self, 'translations'):
            return {}

        has_trans = []
        for trans in self.translations.all():
            has_trans.append(trans.lang)

        return ', '.join(has_trans)

    translation_list.short_description = 'Translated languages'

class InlineTranslations(admin.StackedInline):
    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

class TranslatableAdmin(admin.ModelAdmin):
    init_translations = []
    translation_class = None

    def save_model(self, request, obj, form, change):
        super(TranslatableAdmin, self).save_model(request, obj, form, change)
        self.fix_translations(obj)

    def fix_translations(self, obj):

        if not self.translation_class:
            return

        if not hasattr(obj, 'translations'):
            return

        has_trans = {}
        for trans in obj.translations.all():
            has_trans[trans.lang] = True

        for (lang, lang_name) in settings.LANGUAGES:
            if lang != settings.MAIN_LANGUAGE and not has_trans.has_key(lang):

                fields = {'lang': lang}

                for field_name in self.init_translations:
                    fields[field_name] = translate(getattr(obj, field_name), settings.MAIN_LANGUAGE, lang)

                obj.translations.add(self.translation_class(**fields))

    def get_object(self, request, object_id):
        obj = super(TranslatableAdmin, self).get_object(request, object_id)

        self.fix_translations(obj)

        return obj

class TranslationError(Exception):
    pass

def translate(message, lang_from, lang_to):

    if lang_to not in _languages:
        raise TranslationError, "Language %s is not supported as lang_to." % lang_to
    if lang_from not in _languages and lang_from != '':
        raise TranslationError, "Language %s is not supported as lang_from." % lang_from

    #try:
    #    service = build('translate', 'v2',
    #                    developerKey='AIzaSyBUKnwQMI3r2pIdSxBAjUjmmrKHoHo1q4s')
    #    data = service.translations().list(
    #        source=lang_from,
    #        target=lang_to,
    #        q=[message]
    #    ).execute()
    #
    #    return data['translations'][0]['translatedText']
    #
    #except Exception, e:
    return None



    # detect_url = "http://ajax.googleapis.com/ajax/services/language/detect?v=1.0&q=%(message)s"

_languages = {
  'af': 'Afrikaans',
  'sq': 'Albanian',
  'am': 'Amharic',
  'ar': 'Arabic',
  'hy': 'Armenian',
  'az': 'Azerbaijani',
  'eu': 'Basque',
  'be': 'Belarusian',
  'bn': 'Bengali',
  'bh': 'Bihari',
  'bg': 'Bulgarian',
  'my': 'Burmese',
  'ca': 'Catalan',
  'chr': 'Cherokee',
  'zh': 'Chinese',
  'zh-CN': 'Chinese_simplified',
  'zh-TW': 'Chinese_traditional',
  'hr': 'Croatian',
  'cs': 'Czech',
  'da': 'Danish',
  'dv': 'Dhivehi',
  'nl': 'Dutch',
  'en': 'English',
  'eo': 'Esperanto',
  'et': 'Estonian',
  'fl': 'Filipino',
  'fi': 'Finnish',
  'fr': 'French',
  'gl': 'Galician',
  'ka': 'Georgian',
  'de': 'German',
  'el': 'Greek',
  'gn': 'Guarani',
  'gu': 'Gujarati',
  'iw': 'Hebrew',
  'hi': 'Hindi',
  'hu': 'Hungarian',
  'is': 'Icelandic',
  'id': 'Indonesian',
  'iu': 'Inuktitut',
  'ga': 'Irish',
  'it': 'Italian',
  'ja': 'Japanese',
  'kn': 'Kannada',
  'kk': 'Kazakh',
  'km': 'Khmer',
  'ko': 'Korean',
  'ku': 'Kurdish',
  'ky': 'Kyrgyz',
  'lo': 'Laothian',
  'lv': 'Latvian',
  'lt': 'Lithuanian',
  'mk': 'Macedonian',
  'ms': 'Malay',
  'ml': 'Malayalam',
  'mt': 'Maltese',
  'mr': 'Marathi',
  'mn': 'Mongolian',
  'ne': 'Nepali',
  'no': 'Norwegian',
  'or': 'Oriya',
  'ps': 'Pashto',
  'fa': 'Persian',
  'pl': 'Polish',
  'pt-PT': 'Portuguese',
  'pa': 'Punjabi',
  'ro': 'Romanian',
  'ru': 'Russian',
  'sa': 'Sanskrit',
  'sr': 'Serbian',
  'sd': 'Sindhi',
  'si': 'Sinhalese',
  'sk': 'Slovak',
  'sl': 'Slovenian',
  'es': 'Spanish',
  'sw': 'Swahili',
  'sv': 'Swedish',
  'tg': 'Tajik',
  'ta': 'Tamil',
  'tl': 'Tagalog',
  'te': 'Telugu',
  'th': 'Thai',
  'bo': 'Tibetan',
  'tr': 'Turkish',
  'uk': 'Ukrainian',
  'ur': 'Urdu',
  'uz': 'Uzbek',
  'ug': 'Uighur',
  'vi': 'Vietnamese',
  'cy': 'Welsh',
  'yi': 'Yiddish'
};




#coding:utf-8
# Create your views here.
import base64
import json
from smtplib import SMTPRecipientsRefused
import cloud
from django.contrib import messages
from django.contrib.auth import authenticate, login
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.decorators.csrf import  csrf_exempt
from django.views.decorators.http import require_POST
from django.views.generic.base import TemplateView
from django.utils import translation
from cratis.app.ecommerce.shopping_cart.cart import Cart
from cratis.app.core.common.registration import user_registered_signal
from cratis.app.ecommerce.profile.views import WithShopingCartMixin, WithSidebarMixin, AddressForm
from cratis.app.ecommerce.orders.models import Order, DeliveryMethod, PaymentMethod, Address

from django.utils.translation import ugettext as _
from cratis.app.ecommerce.payment.common import get_client_ip
from cratis.app.core.settings import SHOP_DELIVERY_PER_PRODUCT, APP_SETTINGS, MAIN_LANGUAGE
#from ulink.order import OrderItem as UlinkOrderItem, Order as UlinkOrder
#from ulink.protocol import TransportPacket
#from ulink.request import PaymentRequest, CURRENCY_EURO, create_from_json
#import ulink.utils as CryptoUtils

import logging

from django.contrib.auth.models import User

logger = logging.getLogger('maxe')

import string, random
from cratis.app.core.common.templatetags.model_trans import localize

def generate_password(length):
    password = []

    password_len = length

    for group in (string.ascii_letters, string.digits):
        password += random.sample(group, 3)

    password += random.sample(
        string.ascii_letters + string.punctuation + string.digits,
        password_len - len(password))

    random.shuffle(password)
    return ''.join(password)


@require_POST
def set_delivery(request):
    method = request.POST['method']
    order = Order.objects.get(pk=request.session['order_id'])
    order.delivery_method = DeliveryMethod.objects.get(pk=method)
    order.save()
    return render_to_response('shops/order_details.html', {'details': order.details})

def json_response(response):
    return HttpResponse(json.dumps(response), mimetype='application/javascript')

@require_POST
def validate(request):
    order = Order.objects.get(pk=request.session['order_id'])

    if not order.address:
        return json_response({'ok': False, 'type': 'overall', 'msg': str(_('Address is not selected'))})

    if order.address.temporary:
        address_form = AddressForm(order.address.__dict__, instance=order.address)
        if not address_form.is_valid():
            return json_response({'ok': False, 'type': 'form', 'errors': address_form._errors,
                                  'msg': _('Address is not filled correctly')})

    if not request.user or request.user.is_anonymous():
        result = User.objects.filter(email=order.email)
        if result:
            return json_response({'ok': False, 'type': 'overall', 'msg': _('Email belongs to one of site accounts. Please login to the site to use this email.')})

    return json_response({'ok': True})

@require_POST
def set_payment(request):
    method = request.POST['method']
    order = Order.objects.get(pk=request.session['order_id'])

    #if not request.user or request.user.is_anonymous():
    #    result = User.objects.filter(email=order.email)
    #    if result:
    #        return render_to_response('shops/payment_not_ready.html',
    #                {'details': order.details, 'order': order, 'error': _('Email belongs to one of site accounts. Please login to the site to use this email.')})
    #
    #if not order.address or not order.address.is_ready():
    #    return render_to_response('shops/payment_not_ready.html',
    #        {'details': order.details, 'order': order, 'error': _('You should fill all address fields to continue.')})


    method = PaymentMethod.objects.get(pk=method)
    order.payment_method = method
    order.save()

    return render_to_response('shops/payment_actions.html', {'details': order.details, 'order': order},
                                    context_instance=RequestContext(request))




#@require_POST
#def set_smartpost_box(request):
#    order = Order.objects.get(pk=request.session['order_id'])
#    order.smartpost_box = request.POST['box_id']
#    order.smartpost_box_name = request.POST['box_name']
#    order.save()
#    return HttpResponse(json.dumps({'ok':True}), mimetype='application/javascript')




@require_POST
def update_address(request):
    order = Order.objects.get(pk=request.session['order_id'])

    if 'address_id' in request.POST:
        if order.address and order.address.temporary:
            order.address.delete()

        if request.POST['address_id'] == 0:
            order.address = None
        else:
            try:
                order.address = Address.objects.get(pk=request.POST['address_id'], profile=request.user.get_profile())

                for address in request.user.get_profile().saved_adresses():
                    address.is_main = (order.address.pk == address.pk)
                    address.save()

            except ObjectDoesNotExist:
                order.address = None
        order.save()
    else:
        if not order.address or not order.address.temporary:
            address = Address()
            address.temporary = True
            address.profile = request.user.get_profile() if not request.user.is_anonymous() else None
            address.save()
            order.address = address
            order.save()

        #if 'country' in request.POST:
        #    order.address.country = request.POST['country']
        #order.address.city = request.POST['city']
        #order.address.post_index = request.POST['post_index']
        #order.address.email = request.POST['email']
        #order.address.lastname = request.POST['lastname']
        #order.address.street_address = request.POST['street_address']
        #order.address.firstname = request.POST['firstname']
        #order.address.phone = request.POST['phone']

        address_form = AddressForm(request.POST, instance=order.address)

        if not address_form.is_valid():
            # still save data
            for key, value in request.POST.items():
                setattr(order.address, key, value)
            order.address.save()
            return HttpResponse(json.dumps({'ok': False, 'errors': address_form._errors}), mimetype='application/javascript')
        address_form.save()

    if order.address.email:
        order.email = order.address.email
        order.save()

    return HttpResponse(json.dumps({'ok':True}), mimetype='application/javascript')


@require_POST
def update_contacts(request):
    order = Order.objects.get(pk=request.session['order_id'])

    order.email = request.POST['email']

    order.save()

    return HttpResponse(json.dumps({'ok':True}), mimetype='application/javascript')

class OrderStatusView(WithShopingCartMixin, WithSidebarMixin, TemplateView):
    template_name= 'order_status.html'

    def get_context_data(self, **kwargs):
        context = super(OrderStatusView, self).get_context_data(**kwargs)
        return context

class PaymentWaitView(WithShopingCartMixin, WithSidebarMixin, TemplateView):
    template_name='payment_wait.html'

    def get_context_data(self, **kwargs):
        context = super(PaymentWaitView, self).get_context_data(**kwargs)

        order = Order.objects.get(pk=self.request.session['order_id'])
        context['order'] = order
        context['status'] = order.payment_status if order.payment_status else 'wait'

        return context

class CheckoutDoneView(WithShopingCartMixin, WithSidebarMixin, TemplateView):
    template_name= 'checkout_confirm.html'

    def get_context_data(self, **kwargs):
        context = super(CheckoutDoneView, self).get_context_data(**kwargs)

        context['order'] = Order.objects.get(pk=self.request.session['prev_order_id'])

        return context

class CheckoutView(WithShopingCartMixin, WithSidebarMixin, TemplateView):
    template_name= 'checkout.html'

    def create_new_order(self, request, shopping_cart):
        order = Order()

        if not self.request.user.is_anonymous():
            order.user = self.request.user.get_profile()
            for address in order.user.saved_adresses():
                if address.is_main:
                    order.address = address

        order.status = Order.ORDER_STATUS_NEW
        order.cart = shopping_cart.cart
        if request.session.has_key('prev_order_id'):
            try:
                prev_order = Order.objects.get(pk=request.session['prev_order_id'])

                for field in (
                'email', 'delivery_method', 'payment_method', 'address'):
                    setattr(order, field, getattr(prev_order, field))
            except ObjectDoesNotExist:
                pass

        order.save()
        request.session['order_id'] = order.id

        return order

    def prepare_order(self, request):

        shopping_cart = Cart(request)

        if (not request.session.has_key('order_id')) or request.session['order_id'] is None:
            order = self.create_new_order(request, shopping_cart)
        else:
            try:
                order = Order.objects.get(pk=request.session['order_id'])
            except ObjectDoesNotExist:
                order = self.create_new_order(request, shopping_cart)

        # update order items
        order.fill_items()

        order.ip_address = get_client_ip(request)

        if not order.email and not request.user.is_anonymous():
            order.email = request.user.email
            order.save()

        if not order.user and not request.user.is_anonymous():
            order.user = request.user.get_profile()
            order.save()

        return order

    def get_context_data(self, **kwargs):
        context = super(CheckoutView, self).get_context_data(**kwargs)

        order = self.prepare_order(self.request)

        context['order'] = order

        context['delivery_methods'] = DeliveryMethod.objects.all()
        context['payment_methods'] = PaymentMethod.objects.filter(enabled=True)

        if not self.request.user.is_anonymous():
            context['addresses'] = self.request.user.get_profile().adresses.filter(temporary=False)

        if order.address:
            address_form = AddressForm(instance=order.address)
        else:
            address_form = AddressForm()

        context['address_form'] = address_form

        if len(context['delivery_methods']) == 1:
            order.delivery_method = context['delivery_methods'][0]
            order.save()

        if not order.payment_method and len(context['payment_methods']):
            order.payment_method = context['payment_methods'][0]
            order.save()

        context['need_delivery'] = not SHOP_DELIVERY_PER_PRODUCT

        return context


@require_POST
def payment_check(request):
    order = Order.objects.get(pk=request.session['order_id'])
    return HttpResponse(json.dumps({'status':order.payment_status}), mimetype='application/json')

@require_POST
@csrf_exempt
def payment_response(request):

#    try:
#        rawData = request.POST['signedResponse']
#
#        privateKey = CryptoUtils.load_pem_private_key(ULINK_PRIV_KEY)
#        pubKey = CryptoUtils.load_pem_public_key(ULINK_SERVER_PUB_KEY)
#
#        packet = TransportPacket.create_from_json(rawData)
#        packet.validateAgainstKey(pubKey)
#        decodedPacket = CryptoUtils.unseal(packet.request, privateKey)
#        paymentResponse = create_from_json(smart_unicode(decodedPacket))
#
#        order = Order.objects.get(pk=paymentResponse.clientTransactionId)
#
#        if paymentResponse.isSuccess:
#            order.payment_status = 'success'
#            order.status = Order.ORDER_STATUS_PAID
#        else:
#            order.payment_status = 'failure'
#
#        order.payment_response = smart_unicode(decodedPacket)
#        order.save()
#
#        return HttpResponse(json.dumps({'status':'OK'}), mimetype='application/json')
#
#    except Exception as e:
    return HttpResponse(json.dumps({'status':'NOTOK', 'msg': 'WTF?'}), mimetype='application/json')


def get_url_prefix():
    current_site = Site.objects.get_current()
    prefix = 'http://' + current_site.domain
    return prefix


def get_order_hash_url(order):
    prefix = get_url_prefix()
    order_url = prefix + reverse('orders_status', kwargs={'order_id': order.id, 'hash': order.get_linkhash()})
    return order_url

def add_invoice_delivery_task(order):

    translation.activate(MAIN_LANGUAGE)

    address_post_index = ", " + order.address.post_index if order.address.post_index else ''
    delivery_address = order.address.firstname + ' ' + order.address.lastname + "\n" +\
                       order.address.street_address + "\n" + \
                       order.address.city + address_post_index + ", " + order.address.country + "\n" + \
        "Tel: " + order.address.phone + "\n" + \
        "Email: " + order.email

    rows = []

    for item in order.cart.items.all():
        rows.append({
            'id': item.pk,
            'title': localize(item.product, 'title'),
            'qty': item.quantity,
            'unitPrice': str(item.unit_price),
            'totalPrice': str(item.total_price),
            'deliveryPrice': str(item.delivery_method.delivery_price if item.delivery_method else 0)
        })

    invoice = {
        'id': order.pk,
        'date': str(order.date_created.date()),
        'email': order.email,
        'adminEmail': APP_SETTINGS['invoice']['email'],
        'fromEmail': APP_SETTINGS['invoice']['email'],
        'companyName': _('Inter24 Inc.'),

        'infoHeader':   "",

        'shippingReturnAddress':   _("InterCom Group LTD.\n" +
                        "Inter24\n" +
                        "Parnu mnt 238, 11622\n" +
                        "Tallinn, Harjumaa\n" +
                        "Estonia"),

        'infoFooterCol1': _("Logistikcenter\n" +
                          "InterCom Group AB:\n" +
                          "Parnu mnt. 238\n" +
                          "11622 Tallinn, Estonia\n" +
                          "Reg. 10113432\n" +
                          "Momsnummer SE502071470401"),

        'infoFooterCol2': _("Telefon: (+46) 812 410 605\n" +
                          "E-post: info@inter24.se\n" +
                          "Webbplats: www.inter24.se"),

        'infoFooterCol3': _("Bank: SEB Banken\n" +
                        "Adress: Kungstradgardsg 8,\n" +
                        "10640 Stockholm, Sweden\n" +
                        "Bankkonto: 52771027720\n" +
                        "BankGiro: 737-3459\n" +
                        "IBAN: SE8750000000052771027720\n" +
                        "SWIFT: ESSESESS"),

        'logo': APP_SETTINGS['invoice']['logo'],
	    'billAddress': delivery_address,
        'shippingAddress': delivery_address,
        'subtotal': str(order.cart.without_tax),
        'tax': APP_SETTINGS['tax'],
        'taxTotal': str(order.cart.tax),
        'total': str(order.cart.total_price),

        'rows': rows,

        'texts': {
           'sh_invoice_no': _("INVOICE No. "),
            'sh_invoice_from': _("From:"),
            'sh_invoice_to': _("To:"),
            'invoice_col_id': _("Product ID"),
            'invoice_col_description': _("Description"),
            'invoice_col_delivery': _("Delivery"),
            'invoice_col_quantity': _("Qty"),
            'invoice_col_total': _("Total"),
            'invoice_label': _("INVOICE"),
            'invoice_date_label': _("Date: "),
            'invoice_no_label': _("INVOICE No. "),
            'invoice_bill_to': _("Bill to:"),
            'invoice_ship_to': _("Ship to:"),
            'invoice_row_no_label': _("No."),
            'invoice_notes_label': _("Notes:"),
            'invoice_subtotal': _("Subtotal: "),
            'invoice_tax': _("Tax: "),
            'invoice_tax_total': _("Tax total: "),
            'invoice_total': _("Total: "),
            '$': _("SEK"),
        }
    }

    json_data = json.dumps(invoice, indent=4)
    order.invoiceData = base64.b64encode(json_data)

    order.save()

    jid = cloud.shell.execute('mvn exec:java -Dexec.args="{data}"', {'data': order.invoiceData}, return_file='report.txt', _env='maxe', cwd='/home/picloud/reporter')



#    conn = EuSQSConnection()
#    q = conn.get_queue('invoice')
#
#    if not q:
#        raise Exception('Can not get queue.')
#
#    m = Message()
#    m.set_body()
#    q.write(m)
#    conn.close()

def confirm(request):

    if not request.session.has_key('order_id'):
        return HttpResponseRedirect(reverse('orders_checkout'))

    order = Order.objects.get(pk=request.session['order_id'])
    order_url = get_order_hash_url(order)

    #    if order.status != Order.ORDER_STATUS_PAID:
    #        return HttpResponse('Wating for payment.')

    autouser = not 'autouser' in APP_SETTINGS or APP_SETTINGS['autouser'] is True

    if autouser and request.user.is_anonymous():

        user = User.objects.filter(email=order.email)
        if user.count() < 1:

            password = generate_password(6)
            user = User.objects.create_user(order.email, order.email, password)

            if not user:
                messages.add_message(request, messages.ERROR, _('Can not create a new user account.'))
                return HttpResponseRedirect(reverse('orders_checkout'))

            new_user = authenticate(username=order.email, password=password)
            user_registered_signal.send(sender=None, user=new_user,
                                        request=request)

            login(request, new_user)

            try:
                message = _(u"Hello, here is your new account for inter24.se You can use it to check your order status.\n\n  Username: %(username)s \n  Password: %(password)s \n\n\nInter24 team") % {'username': order.email, 'password': password }
                subject, from_email, to = _(u'Your new inter24.se account details'), 'info@inter24.se', order.email
                send_mail(subject, message, from_email, [to])

                messages.add_message(request, messages.INFO, _('We have created a new account for you. \n\n  Username: %(username)s\n  Password: %(password)s. You can use it to check order status.') % {'username': order.email, 'password': password })
            except SMTPRecipientsRefused:
                messages.add_message(request, messages.ERROR, _('Can not send invoice to providen email address, please check it.'))
                return HttpResponseRedirect(reverse('orders_checkout'))
        else:
            messages.add_message(request, messages.ERROR, _('User with this username already exists. PLease login first.'))
            return HttpResponseRedirect(reverse('orders_checkout'))

    shopping_cart = Cart(request)
    shopping_cart.cart.checked_out = True
    shopping_cart.cart.order = order
    shopping_cart.cart.save()

    if autouser:
        order.user = request.user.get_profile()

    order.cart = shopping_cart.cart
    order.mark_confirmed()

    if autouser:
        add_invoice_delivery_task(order)

        if order.address.temporary:
            order.address.temporary = False
        order.address.profile = request.user.get_profile()
        order.address.save()

    request.session['order_id'] = None
    request.session['prev_order_id'] = order.id

    if 'success_url' in request.session:
        return HttpResponseRedirect(request.session['success_url'])
    else:
        return HttpResponseRedirect(reverse('orders_checkout_done'))





from cratis.features import Feature, require


class Filer(Feature):
    """
    Pillow installation

    sudo apt-get install python-dev libjpeg-dev libfreetype6-dev zlib1g-dev

    $ sudo ln -s /usr/lib/`uname -i`-linux-gnu/libfreetype.so /usr/lib/
    $ sudo ln -s /usr/lib/`uname -i`-linux-gnu/libjpeg.so /usr/lib/
    $ sudo ln -s /usr/lib/`uname -i`-linux-gnu/libz.so /usr/lib/
    """

    def get_required_packages(self, cls):
        return (
            'easy-thumbnails',
            'Pillow',
            'django-filer'
        )


    def configure_settings(self, cls):

        self.append_apps(cls, [
            'easy_thumbnails',
            'filer',
            'mptt'
        ])

        cls.THUMBNAIL_PROCESSORS = (
            'easy_thumbnails.processors.colorspace',
            'easy_thumbnails.processors.autocrop',
            #'easy_thumbnails.processors.scale_and_crop',
            'filer.thumbnail_processors.scale_and_crop_with_subject_location',
            'easy_thumbnails.processors.filters',
        )

        cls.THUMBNAIL_FORMAT = 'PNG'

        cls.FILER_DEBUG = cls.DEBUG



@require(Filer)
class FilerCms(Feature):

    def get_required_packages(self, cls):
        return 'cmsplugin-filer',


    def configure_settings(self, cls):

        self.append_apps(cls, [
            'cmsplugin_filer_file',
            'cmsplugin_filer_folder',
            'cmsplugin_filer_image',
            'cmsplugin_filer_teaser',
            'cmsplugin_filer_video',
        ])


from cratis.app.ecommerce.payment.common import Payment
from cratis.app.ecommerce.payment.dibs.payment import DibsPayment
from cratis.app.ecommerce.payment.pangalink.payment import PangalinkPayment
from cratis.app.ecommerce.payment.paypal.payment import PaymentPaypal
from cratis.app.ecommerce.payment.kardikeskus.payment import KardikeskusPayment
from cratis.app.ecommerce.payment.paytrail.payment import PaytrailPayment
from cratis.app.ecommerce.payment.voucher.payment import VoucherPayment

__author__ = 'Alex'

from django.utils.translation import ugettext as _


class UnknownDeliveryMethod(Exception):
    pass


def get_payment_methods():
    return {
        'kardikeskus': KardikeskusPayment,
        'pangalink': PangalinkPayment,
        'paypal': PaymentPaypal,
        'paytrail': PaytrailPayment,
        'dibs': DibsPayment,
        'invoice': Invoice,
        'voucher': VoucherPayment,
    }


def get_payment_method(name, config, keys):
    methods = get_payment_methods()
    if name in methods:
        return methods[name](config, keys)
    else:
        raise UnknownDeliveryMethod()


def get_payment_method_selection():
    return (
        ('kardikeskus', _('Kardikeskus payment')),
        ('pangalink', _('Pangalink')),
        ('paytrail', _('Paytrail payment')),
        ('paypal', _('PayPal payment')),
        ('dibs', _('DIBS payment')),
        ('invoice', _('Invoice payment')),
        ('voucher', _('Voucher payment')),
    )


class Invoice(Payment):
    actions_template = 'payment/invoice.html'





from voluptuous import Schema
from cratis.app.ecommerce.payment.common import Payment

__author__ = 'alex'


class PaymentPaypal(Payment):

    actions_template = 'payment/paypal_actions.html'

    def schema(self):
        #TEST = settings.PAYPAL_TEST
        #USER = settings.PAYPAL_WPP_USER
        #PASSWORD = settings.PAYPAL_WPP_PASSWORD
        #SIGNATURE = settings.PAYPAL_WPP_SIGNATURE

        return Schema({
            'USER': str,
            'PASSWORD': str,
            'SIGNATURE': str,
            'TEST': bool,
        })

    def require_payment(self):
        return True


from cms.models.fields import PageField
from django.db import models
from django.db.models.fields import DateTimeField

from django.utils.translation import ugettext_lazy as _
from filer.fields.image import FilerImageField
from cratis.app.i18n.translate import TranslatableModelMixin


class Banner(TranslatableModelMixin, models.Model):
    date_created = DateTimeField(auto_now_add=True)
    date_updated = DateTimeField(auto_now=True)

    name = models.CharField(max_length=255)

    def __unicode__(self):
        return self.name + ', id: ' + str(self.id)

    class Meta:
        verbose_name = _('Banner')
        verbose_name_plural = _('Banners')


class BannerImage(models.Model):
    #    title = models.CharField(max_length=100, blank=True)
    image = FilerImageField(null=True, blank=True)
    banner = models.ForeignKey(Banner)
    title = models.CharField(max_length=100, blank=True, default='', null=False)
    page = PageField(blank=True, null=True)
    url = models.CharField(max_length=100, blank=True, null=True)

    def get_url(self):
        if self.url:
            return self.url
        else:
            return '#'

    class Meta:
        verbose_name = _('Slide')
        verbose_name_plural = _('Slides')



from django.conf import settings
from django.utils import translation

__author__ = 'alex'

from django import template

register = template.Library()

@register.filter
def localize_value(str):
    cur_language = translation.get_language()
#    langs = dict(LANGUAGES)
#    langs.keys()
    vals = str.split(';')
    if len(vals) < 2:
        return vals[0]

    if cur_language == settings.MAIN_LANGUAGE:
        return vals[0]
    else:
        return vals[1]



@register.filter
def localize(obj, field_name):
    if not hasattr(obj, field_name):
        return None

    cur_language = translation.get_language()
    if cur_language == settings.MAIN_LANGUAGE:
        return getattr(obj, field_name)

    found = ''
    for trans in obj.translations.all():
        if trans.lang == cur_language:
            found = getattr(trans, field_name)
            break
    if found and len(found.strip()) > 0:
        return found

    if settings.USE_FALLBACK_TRANSLATION:
        return getattr(obj, field_name)
    else:
        return ""


from django.contrib import admin
from cratis.app.content.models import BannerImage, Banner


class InlineBannerImage(admin.StackedInline):
    model = BannerImage
    extra = 3
    classes = ['collapse']


class BannerAdmin(admin.ModelAdmin):
    inlines = [InlineBannerImage]
    list_display = ('name',)

admin.site.register(Banner, BannerAdmin)



from django.forms import TextInput
from mptt.admin import MPTTModelAdmin
from suit.admin import SortableTabularInline, SortableModelAdmin
from cratis.app.utils.admin import default_overrides, fix_translations
from cratis.app.i18n.translate import TranslatableAdmin, InlineTranslations
from cratis.app.i18n.utils import I18nLabel, message_wrapper
from django.contrib import admin
from models import Product, ProductCategory, ProductImage, ProductTranslation, ProductAttributeFilter
from cratis.app.ecommerce.products.models import ProductCategoryTranslation, ProductAttribute, ProductAttributeType, ProductAttributeValue, ProductAttributeTypeTranslation


class InlineProductCategoryTranslation(InlineTranslations):
    model = ProductCategoryTranslation


class ProductCategoryAdmin(TranslatableAdmin, MPTTModelAdmin, SortableModelAdmin):

    formfield_overrides = default_overrides
    inlines = [InlineProductCategoryTranslation]
    init_translations = ['name', 'description']
    translation_class = ProductCategoryTranslation
    fields = ('name', 'description', 'slug', 'image', 'parent', 'attribute_types', 'allow_filtering')

    list_display = ('name', 'translation_list', 'is_active')
    list_editable = ('is_active',)
    actions = [fix_translations]

    search_fields = ('name', 'slug', '=id')

    sortable = 'order_index'
    mptt_level_indent = 20


class InlineProductImage(SortableTabularInline):
    model = ProductImage
    extra = 2
    classes = ['collapse']

    suit_classes = 'suit-tab suit-tab-images'

    sortable = 'sorting'




class InlineProductTranslation(InlineTranslations):
    model = ProductTranslation
    suit_classes = 'suit-tab suit-tab-translations'
    formfield_overrides = default_overrides


class InlineProductAttributeTypeTranslation(InlineTranslations):
    model = ProductAttributeTypeTranslation


def set_featured(modeladmin, request, queryset):
    queryset.update(featured=True)


set_featured.short_description = "Set featured"


def make_active(modeladmin, request, queryset):
    queryset.update(active=True)


make_active.short_description = "Set active"


def make_inactive(modeladmin, request, queryset):
    queryset.update(active=False)


make_inactive.short_description = "Set inactive"


def reset_featured(modeladmin, request, queryset):
    queryset.update(featured=False)


reset_featured.short_description = "Clear featured"


def set_not_sold(modeladmin, request, queryset):
    queryset.update(sold=False)


set_not_sold.short_description = "Set not sold"


def set_sold(modeladmin, request, queryset):
    queryset.update(sold=True)
set_sold.short_description = "Set sold"


def set_archived(modeladmin, request, queryset):
    queryset.update(archived=True, active=False, slug='')
set_archived.short_description = "Remove (archive) products"


class InlineAttribute(admin.TabularInline):

    formfield_overrides = default_overrides
    model = ProductAttribute
    extra = 1

    suit_classes = 'suit-tab suit-tab-attr'

    def formfield_for_manytomany(self, db_field, request=None, **kwargs):
        if db_field.name == "values":
            kwargs["queryset"] = ProductAttributeValue.objects.filter(type=self.model._get_pk_val())
        return super(InlineAttribute, self).formfield_for_manytomany(db_field, request, **kwargs)


class ProductAttributeTypeAdmin(TranslatableAdmin, admin.ModelAdmin):

    formfield_overrides = default_overrides
    inlines = [InlineProductAttributeTypeTranslation]
    ordering = ['name']

    fields = ('name',)
    init_translations = ['name']
    translation_class = ProductAttributeTypeTranslation
    actions = [fix_translations, ]

    search_fields = ('name', )


class ProductAdmin(TranslatableAdmin, admin.ModelAdmin):
    inlines = [InlineAttribute, InlineProductTranslation, InlineProductImage]
    init_translations = ['title', 'long_title', 'description', 'long_description']
    translation_class = ProductTranslation
    formfield_overrides = default_overrides

    def queryset(self, request):
        return super(ProductAdmin, self).queryset(request).filter(archived=False)


    list_filter = ('active', 'categories', 'sold', 'featured')
    list_display = ('title', 'long_title', 'id', 'slug_list', 'price', 'category_list', 'active', 'sold', 'featured')

    search_fields = ('title', '=id', 'categories__name')

    actions = [fix_translations, set_featured, reset_featured, set_sold, set_not_sold, make_active, make_inactive, set_archived]

    fieldsets = (
        (None, {
            'classes': ('suit-tab suit-tab-general',),
            'fields': (
                'title', 'long_title', 'slug', 'description', 'long_description', 'categories')
        }),
        ('Avalibility', {
            'classes': ('suit-tab suit-tab-general',),
            'fields': (
                'sold', 'active')
        }),
        ('Pricing', {
            'classes': ('suit-tab suit-tab-pricing',),
            'fields': ('price', 'price_discount', 'discount')
        }),
        ('Promotion', {
            'classes': ('suit-tab suit-tab-pricing',),
            'fields': ('related', 'featured', 'sold_count', )
        })
    )

    suit_form_tabs = (
        ('general', 'General'),
        ('pricing', 'Pricing & Promotion'),
        ('attr', 'Attributes'),
        ('translations', 'Translations'),
        ('images', 'Pictures'),
    )

    # suit_form_includes = (
    #     ('admin/trans_info.html', 'top', 'translations'),
    # )

    # class Media:
    #     js = (
            # "js/admin/product_form.js",
            # "js/admin/tiny_mce/tiny_mce.js",
            # "js/admin/tinymce_init.js",
            # '/media/admin/custom/js/inlinecollapsed.js',
        # )


admin.site.register(Product, ProductAdmin)
admin.site.register(ProductAttributeType, ProductAttributeTypeAdmin)
admin.site.register(ProductAttributeFilter)
admin.site.register(ProductCategory, ProductCategoryAdmin)

# admin.site.register = I18nLabel(admin.site.register).register()
# admin.site.app_index = I18nLabel(admin.site.app_index).index()
admin.ModelAdmin.message_user = message_wrapper(admin.ModelAdmin.message_user)


from cratis.features import Feature, require
#
#from cms.appresolver import applications_page_check
#
#
#class LazyPage(object):
#    def __get__(self, request, obj_type=None):
#        from cms.utils.page_resolver import get_page_from_request
#        if not hasattr(request, '_current_page_cache'):
#            request._current_page_cache = get_page_from_request(request)
#            if not request._current_page_cache:
#                # if this is in a apphook
#                # find the page the apphook is attached to
#                request._current_page_cache = applications_page_check(request)
#        return request._current_page_cache
#
#
#class CurrentPageMiddleware(object):
#    def process_request(self, request):
#        request.__class__.current_page = LazyPage()
#        return None

from cratis.features.i18n import I18n


@require(I18n)
class Cms(Feature):
    def get_required_packages(self, cls):
        return ('django-mptt',
                'django-cms',
                'Pillow',
                #'django-filer',
                #'cmsplugin-filer',

                #'easy_thumbnails',
                )

    def configure_settings(self, cls):
        self.append_apps(cls, [
            'cms',
            'mptt',
            'menus',
            'south',
            'sekizai',

            'cms.plugins.flash',
            'cms.plugins.googlemap',
            'cms.plugins.link',
            'cms.plugins.text',
            'cms.plugins.twitter'
        ])

        self.append_middleware(cls, [
            'cms.middleware.page.CurrentPageMiddleware',
            'cms.middleware.user.CurrentUserMiddleware',
            'cms.middleware.toolbar.ToolbarMiddleware',
            'cms.middleware.language.LanguageCookieMiddleware'
        ])

        self.append_template_processor(cls, [
            'cms.context_processors.media',
            'sekizai.context_processors.sekizai'
        ])

        cls.CMS_TEMPLATES = ()


    def configure_urls(self, cls, urls):

        from django.conf.urls import url, patterns
        from cms.views import details
        from django.views.decorators.cache import cache_page
        from cratis.app.i18n.utils import localize_url as _

        if cls.DEBUG:
            urls += patterns('',
                             url(_(r'^$'), details, {'slug': ''}, name='pages-root'),
                             url(_(r'^(?P<slug>[0-9A-Za-z-_.//]+)$'), details, name='pages-details-by-slug'),
            )
        else:
            urls += patterns('',
                             url(_(r'^$'), cache_page(60 * 24)(details), {'slug': ''}, name='pages-root'),
                             url(_(r'^(?P<slug>[0-9A-Za-z-_.//]+)$'), cache_page(60 * 24)(details),
                                 name='pages-details-by-slug'),
            )



from django.utils.encoding import smart_str
from django.utils.safestring import mark_safe
import re
import locale


__author__ = 'Alex'

from django import template
register = template.Library()


VALID_TAGS = ['strong', 'em', 'p', 'ul', 'li', 'br', 'img']

from cms.models import Page
from cms.templatetags.cms_tags import PageUrl



class PageUrlOr(PageUrl):
    name = 'page'

    def get_context(self, context, page_lookup, lang, site):
        try:
            context = super(PageUrlOr, self).get_context(context, page_lookup, lang, site)
        except Page.DoesNotExist:
            return {'content': '#'}
        return context

register.tag(PageUrlOr)

@register.filter()
def smartstr(val):
    return smart_str(val)

@register.filter()
def is_category_active(node, current):
    return node.id == current.id or (node.parent and node.parent.id == current.id or (node.parent.parent and node.parent.parent.id == current.id))

@register.filter()
def nospaces(value):
    print(value)
    value = re.sub('/\s+/m', ' ', value)
    print value
    return value

@register.filter
def get(d, key):
    return d.get(key)




__author__ = 'Alex'

from django.utils.translation import ugettext as _

class UnknownDeliveryMethod(Exception):
    pass

def get_delivery_methods():
    return {
        'post': DeliveryPost()
    }

def get_delivery_method(name):
    methods = get_delivery_methods()
    if name in methods:
        return methods[name]
    else:
        raise UnknownDeliveryMethod()

def get_delivery_method_selection():
    return (
        ('post', _('Post-like delivery to address')),
    )


class NotImplemented(Exception):
    pass

class Delivery(object):

    def get_display_label(self):
        raise NotImplemented

    def require_address(self):
        return False


class DeliveryPost(Delivery):

    def require_address(self):
        return True




from django.shortcuts import get_object_or_404
from django.views.generic.base import ContextMixin, View
from cratis.app.ecommerce.payment.models import PaymentMethod


class PaymentView(View):

    def load_method(self, method):
        return  get_object_or_404(PaymentMethod, slug=method)


from django.conf.urls.defaults import patterns, include, url

urlpatterns = patterns('',
    (r'^ipn/notify/8372983hfjewhfew83/', include('paypal.standard.ipn.urls')),
)

urlpatterns += patterns('cratis.app.ecommerce.payment.paypal.views',
    url(r'^(?P<method>[^\\]+)/start', 'payment_start', name='paypal_payment_start'),
    url(r'^(?P<method>[^\\]+)/confirm', 'payment_confirm', name='paypal_payment_confirm'),
)





from decimal import Decimal
from django.db import models
from django.db.models.deletion import PROTECT
from django.utils.translation import ugettext_lazy as _
from filer.fields.image import FilerImageField
from cratis.app.core.common.translate import TranslatableModelMixin
from cratis.app.core.settings import APP_SETTINGS
from cratis.app.ecommerce.delivery.methods import get_delivery_method_selection, get_delivery_method
from cratis.app.ecommerce.products.models import Product


class DeliveryMethodTranslation(models.Model):
    lang = models.CharField(max_length=2, editable=False)
    related = models.ForeignKey('DeliveryMethod', null=True, related_name='translations')
    name = models.CharField(max_length=150, blank=True, default='')
    description = models.TextField(blank=True, default='')

    class Meta:
        app_label = 'orders'


class DeliveryMethod(TranslatableModelMixin, models.Model):
    name = models.CharField(max_length=150, unique=True)
    #logo = FilerImageField(null=True, blank=True)
    delivery_price = models.DecimalField(max_digits=18, decimal_places=2, verbose_name=_('delivery price'))
    description = models.TextField(blank=True, default='')
    method = models.CharField(max_length=20, choices=get_delivery_method_selection(), null=True)
    enabled = models.BooleanField(default=False, verbose_name=_('Enabled'))

    def behavior(self):
        return get_delivery_method(self.method)

    @staticmethod
    def get_translation_class():
        return DeliveryMethodTranslation

    class Meta:
        app_label = 'orders'

    def __unicode__(self):
        return self.name


class Cart(models.Model):
    creation_date = models.DateTimeField(verbose_name=_('creation date'))
    checked_out = models.BooleanField(default=False, verbose_name=_('checked out'))

    order = models.ForeignKey('orders.Order', null=True, related_name='_cart')

    class Meta:
        verbose_name = _('cart')
        verbose_name_plural = _('carts')
        ordering = ('-creation_date',)

    def __unicode__(self):
        return unicode(self.creation_date)

    def total_price(self):
        items = Item.objects.filter(cart=self)
        total = Decimal('0.0')
        for item in items:
            total += item.total_price
        return total

    total_price = property(total_price)

    def tax(self):
        return Decimal(float(self.total_price) - (float(self.total_price) / (1.0 + float(APP_SETTINGS['tax']) / 100.0)))

    tax = property(tax)

    def without_tax(self):
        return self.total_price - self.tax

    without_tax = property(without_tax)


class Item(models.Model):
    cart = models.ForeignKey(Cart, verbose_name=_('cart'), related_name='items')
    quantity = models.PositiveIntegerField(verbose_name=_('quantity'))
    unit_price = models.DecimalField(max_digits=18, decimal_places=2, verbose_name=_('unit price'))

    delivery_method = models.ForeignKey(DeliveryMethod, null=True, on_delete=PROTECT)

    order = models.ForeignKey('orders.Order', null=True)

    product = models.ForeignKey(Product, null=True, on_delete=PROTECT)

    class Meta:
        verbose_name = _('item')
        verbose_name_plural = _('items')
        ordering = ('cart',)

    def __unicode__(self):
        return u'%d units of %s' % (self.quantity, self.product.__class__.__name__)

    def total_price(self):
        price = self.quantity * self.unit_price

        if self.delivery_method:
            price += self.delivery_method.delivery_price * self.quantity

        return price

    total_price = property(total_price)

